﻿Imports System.IO
Imports System.Drawing
Imports Telerik.Windows.Data
Imports GlucoGuardian.Business.Logic.Objects

Class Configuration

    Dim isNewUser As Boolean = True

    Private Sub Configuration_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        Try

            VerifyUserType()

            'Carreguem les capçaleres personalitzades dels informes

            txtHeader1.Text = Main.GetVariable("General", "Header1")
            txtHeader2.Text = Main.GetVariable("General", "Header2")

            If Main.logedUser.USE_logo IsNot Nothing Then

                Dim bi As New BitmapImage
                Dim img As System.Drawing.Image
                Dim ms As New MemoryStream

                img = Bytes2Image(Main.logedUser.USE_logo)

                bi.BeginInit()
                img.Save(ms, System.Drawing.Imaging.ImageFormat.Bmp)
                ms.Seek(0, SeekOrigin.Begin)
                bi.StreamSource = ms
                bi.EndInit()

                imgLogo.Source = bi

            End If

            'Carreguem els valors de Low, Normal i High color

            Dim strLowcolor As String = Main.GetVariable("General", "LowColor")
            Dim strNormalColor As String = Main.GetVariable("General", "NormalColor")
            Dim strHighColor As String = Main.GetVariable("General", "HighColor")

            colLow.SelectedColor = HexToRgb(strLowcolor)
            lblLowColor.Background = New SolidColorBrush(HexToRgb(strLowcolor))
            colNormal.SelectedColor = HexToRgb(strNormalColor)
            lblNormalColor.Background = New SolidColorBrush(HexToRgb(strNormalColor))
            colHigh.SelectedColor = HexToRgb(strHighColor)
            lblHighColor.Background = New SolidColorBrush(HexToRgb(strHighColor))

            'Carreguem l'interval de dies amb el que s'inicia la pestanya de resultats

            txtShowLast.Text = Main.GetVariable("General", "ShowLast")

            'Carreguem la llista de períodes en el grid

            Dim descriptor As New SortDescriptor()
            descriptor.Member = "PER_ticks"
            descriptor.SortDirection = ComponentModel.ListSortDirection.Ascending
            griPeriods.SortDescriptors.Add(descriptor)

            griPeriods.ItemsSource = Main.lstPeriods

            'Carreguem la llista de proves i els seus rangs

            cmbTest.ItemsSource = Main.obj_DAO.GetTests("", "")
            cmbTest.DisplayMemberPath = "TES_name"
            cmbTest.Text = "GLU"

            GetTestRanges()

            'Carreguem la llista d'usuaris

            griUsers.ItemsSource = Main.obj_DAO.GetUsers("", "")

            cmbType.Items.Clear()
            cmbType.Items.Add(Main.UserType.ADMIN.ToString)
            cmbType.Items.Add(Main.UserType.ENDOCRINE.ToString)
            cmbType.Items.Add(Main.UserType.LAB.ToString)
            cmbType.Items.Add(Main.UserType.MANAGER.ToString)
            cmbType.SelectedIndex = 0

            GetAvailableSections()

            'Si l'usuari és administrador carreguem els logs per a que els pugui consultar

            If Main.logedUser.USE_type = Main.UserType.ADMIN.ToString Then
                datFrom.SelectedDate = Now()
                GetLogs()
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub VerifyUserType()

        'Segons el tipus d'usuari li permetem veure unes pestanyes o altres

        Try

            Select Case Main.logedUser.USE_type

                Case Main.UserType.ADMIN.ToString

                    'L'administrador només té accés a la configuració d'usuaris i consulta de logs

                    tabGeneral.Visibility = Windows.Visibility.Collapsed
                    tabTests.Visibility = Windows.Visibility.Collapsed
                    tabUsers.Visibility = Windows.Visibility.Visible
                    tabLogs.Visibility = Windows.Visibility.Visible
                    tabUsers.IsSelected = True

                Case Main.UserType.ENDOCRINE.ToString

                    'L'endocrí té acceés a la seva configuració general i de proves

                    tabGeneral.Visibility = Windows.Visibility.Visible
                    tabTests.Visibility = Windows.Visibility.Visible
                    tabUsers.Visibility = Windows.Visibility.Collapsed
                    tabLogs.Visibility = Windows.Visibility.Collapsed
                    tabGeneral.IsSelected = True

                Case Main.UserType.LAB.ToString

                    'El responsable del laboratori té acceés a la seva configuració general i de proves

                    tabGeneral.Visibility = Windows.Visibility.Visible
                    tabTests.Visibility = Windows.Visibility.Visible
                    tabUsers.Visibility = Windows.Visibility.Collapsed
                    tabLogs.Visibility = Windows.Visibility.Collapsed
                    tabGeneral.IsSelected = True

                Case Main.UserType.MANAGER.ToString

                    'El gerent de l'hospital té accés a la seva configuració general i de proves

                    tabGeneral.Visibility = Windows.Visibility.Visible
                    tabTests.Visibility = Windows.Visibility.Visible
                    tabUsers.Visibility = Windows.Visibility.Collapsed
                    tabLogs.Visibility = Windows.Visibility.Collapsed
                    tabGeneral.IsSelected = True

            End Select


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub RefreshUserConfig()

        Main.userConfig = Main.obj_DAO.GetConfigurations("GG_configuration.USE_id_user =" & Main.logedUser.USE_id_user, "")

    End Sub

    Private Sub imgLogo_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles imgLogo.MouseDown

        Try

            Dim dlg As New Microsoft.Win32.OpenFileDialog()

            dlg.FileName = "" ' Nombre por defecto del archivo

            ' Abrimos un FILE DIALOG
            Dim result As Boolean = dlg.ShowDialog()

            If result = True Then

                ' Abrimos el archivo
                Dim filename As String = dlg.FileName

                Dim logo As BitmapImage = New BitmapImage()
                logo.BeginInit()
                logo.UriSource = New Uri(filename, UriKind.Absolute)
                logo.EndInit()

                imgLogo.Source = logo

                'Convertimos la imagen en un array de bytes y la insertamos en la bd
                Dim ms As New MemoryStream
                Dim bbe As New BmpBitmapEncoder()
                bbe.Frames.Add(BitmapFrame.Create(New Uri(imgLogo.Source.ToString(), UriKind.RelativeOrAbsolute)))
                bbe.Save(ms)

                Dim img2 As System.Drawing.Image = System.Drawing.Image.FromStream(ms)

                Main.logedUser.USE_logo = Image2Bytes(img2)

                Main.obj_DAO.InsertUserLogo(Main.logedUser)

                Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "User logo changed")

            End If


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub txtHeader1_LostFocus(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles txtHeader1.LostFocus

        'Si el textbox perd el focus actualitzem la BD per si s'ha modificat el contingut

        Try

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & txtHeader1.Text & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='General' AND CFG_variable='Header1'")
            RefreshUserConfig()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Header1 changed to " & txtHeader1.Text)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub txtHeader2_LostFocus(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles txtHeader2.LostFocus

        'Si el textbox perd el focus actualitzem la BD per si s'ha modificat el contingut

        Try

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & txtHeader2.Text & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='General' AND CFG_variable='Header2'")
            RefreshUserConfig()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Header2 changed to " & txtHeader2.Text)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub


    Private Sub colLow_SelectedColorChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles colLow.SelectedColorChanged

        'Si canviem el color del selector de colorts actualitzem la BD

        Try

            lblLowColor.Background = New SolidColorBrush(colLow.SelectedColor)

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & colLow.SelectedColor.ToString & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='General' AND CFG_variable='LowColor'")
            RefreshUserConfig()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Low color changed to " & colLow.SelectedColor.ToString)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub colHigh_SelectedColorChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles colHigh.SelectedColorChanged

        'Si canviem el color del selector de colorts actualitzem la BD

        Try

            lblHighColor.Background = New SolidColorBrush(colHigh.SelectedColor)

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & colHigh.SelectedColor.ToString & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='General' AND CFG_variable='HighColor'")
            RefreshUserConfig()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "High color changed to " & colHigh.SelectedColor.ToString)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub colNormal_SelectedColorChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles colNormal.SelectedColorChanged

        'Si canviem el color del selector de colorts actualitzem la BD

        Try

            lblNormalColor.Background = New SolidColorBrush(colNormal.SelectedColor)

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & colNormal.SelectedColor.ToString & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='General' AND CFG_variable='NormalColor'")
            RefreshUserConfig()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Normal color changed to " & colNormal.SelectedColor.ToString)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub txtShowLast_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Input.KeyEventArgs) Handles txtShowLast.KeyDown

        'Amb aquesta funció ens assegurem que només es poden introduir números en el textbox

        Try

            If (e.Key >= Key.D0 And e.Key <= Key.D9) Or (e.Key >= Key.NumPad0 And e.Key <= Key.NumPad9) Then
                e.Handled = False
            Else
                e.Handled = True
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub txtShowLast_LostFocus(sender As Object, e As System.Windows.RoutedEventArgs) Handles txtShowLast.LostFocus

        'Si el textbox perd el focus actualitzem la BD per si s'ha modificat el contingut

        Try

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & txtShowLast.Text & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='General' AND CFG_variable='ShowLast'")
            RefreshUserConfig()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Show last days changed to " & txtShowLast.Text)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub griPeriods_RowEditEnded(ByVal sender As Object, ByVal e As Telerik.Windows.Controls.GridViewRowEditEndedEventArgs) Handles griPeriods.RowEditEnded

        'Quan acabem d'editar les hores d'un període actualtzem la configuració a la BD

        Dim strHour As String = ""
        Dim idPeriod As Integer = 0

        Try

            strHour = DirectCast(griPeriods.SelectedItem, GG_periods).PER_start_hour

            idPeriod = DirectCast(griPeriods.SelectedItem, GG_periods).PER_id_period

            If idPeriod < Main.lstPeriods.Count Then
                ModifyPeriod(idPeriod, Mid(strHour, 1, 5))
                UpdatePeriods()
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub RadTimePicker_SelectedTimeChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)

        'Quan acabem d'editar les hores d'un període actualtzem la configuració a la BD

        Dim strHour As String = ""
        Dim idPeriod As Integer = 0

        Try

            strHour = DirectCast(sender, Telerik.Windows.Controls.RadTimePicker).SelectedTime.ToString

            idPeriod = DirectCast(DirectCast(sender, Telerik.Windows.Controls.RadTimePicker).DataContext, GG_periods).PER_id_period

            If idPeriod < Main.lstPeriods.Count Then

                ModifyPeriod(idPeriod, Mid(strHour, 1, 5))
                UpdatePeriods()

            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Sub ModifyPeriod(ByVal perId As Integer, ByVal perHour As String)

        Dim lastId As Integer
        Dim strStartHour As String

        Try

            'Busquem algun període amb la mateixa hora, si hi és no fem la modificació

            Dim counter = (From p In Main.lstPeriods _
                           Where p.PER_start_hour Like perHour _
                           And p.PER_id_period <> perId).Count

            If counter > 0 Then

                Main.ShowMessage(My.Resources.Resource.msg_PeriodExists)
                griPeriods.Rebind()
                Exit Sub

            End If

            'Actualitzem el període actual
            Main.lstPeriods.Item(perId).PER_start_hour = Mid(perHour, 1, 5)

            Dim tempList = From p In Main.lstPeriods _
                           Order By p.PER_start_hour Descending

            lastId = tempList(0).PER_id_period
            strStartHour = tempList(0).PER_start_hour

            For Each per In tempList

                If strStartHour <> "" Then
                    Main.lstPeriods.Item(per.PER_id_period).PER_end_hour = strStartHour
                End If

                strStartHour = per.PER_start_hour

            Next

            Main.lstPeriods.Item(lastId).PER_end_hour = strStartHour

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Sub UpdatePeriods()

        'Actualitzem el valor dels períodes en la BD

        Dim strPeriodsMounted As String

        Try

            strPeriodsMounted = MountPeriodsList()

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & strPeriodsMounted & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='General' AND CFG_variable='Periods'")
            RefreshUserConfig()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Periods changed to " & strPeriodsMounted)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Function MountPeriodsList() As String

        'Amb aquesta funció preparem la llista de períodes abans d'insertar-la a la BD

        Dim strPeriods As String = ""

        Try

            Dim tempList = From p In Main.lstPeriods Select p Order By p.PER_start_hour Ascending

            For Each per In tempList

                strPeriods = strPeriods & per.PER_start_hour & "-" & per.PER_end_hour & ";" & per.PER_name & "|"

            Next

            strPeriods = Mid(strPeriods, 1, Len(strPeriods) - 1)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return strPeriods

    End Function

    Private Sub btnRemovePeriod_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnRemovePeriod.Click

        'Podem borrar períodes de la llista (n'ha de quedar mínim 1)

        Try

            If Main.lstPeriods.Count = 1 Or griPeriods.SelectedItem Is Nothing Then

                Exit Sub

            End If

            Dim idPeriod As Integer = 0

            idPeriod = DirectCast(griPeriods.SelectedItem, GG_periods).PER_id_period

            Main.lstPeriods.RemoveAt(idPeriod)

            griPeriods.ItemsSource = Main.lstPeriods

            Dim counter As Integer = 0

            For Each p In Main.lstPeriods

                p.PER_id_period = counter
                counter = counter + 1

            Next

            UpdatePeriods()
            griPeriods.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnAddPeriod_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnAddPeriod.Click

        'Amb aquest botó es poden afegir períodes (límit de 8)

        Try

            If Main.lstPeriods.Count = 8 Then

                Main.ShowMessage(My.Resources.Resource.msg_PeriodsMax)
                Exit Sub

            End If

            Dim p As New GG_periods

            p.PER_id_period = Main.lstPeriods.Count
            p.PER_start_hour = Nothing
            p.PER_end_hour = Nothing

            Main.lstPeriods.Add(p)

            UpdatePeriods()
            griPeriods.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub GetTestRanges()

        'Carreguem els rangs d'una prova determinada (GLU,KET,HB)

        Dim strTest As String = cmbTest.Text

        Try

            txtMin.Text = Main.GetVariable("Test", strTest & "_min")
            txtMax.Text = Main.GetVariable("Test", strTest & "_max")

            InitializeRanges(Main.GetVariable("Test", strTest & "_ranges"))
            griRanges.ItemsSource = Main.lstRanges

            griRanges.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbTest_DropDownClosed(ByVal sender As Object, ByVal e As System.EventArgs) Handles cmbTest.DropDownClosed

        'Si canviem la prova en el combobox recarreguem els rangs

        Try

            GetTestRanges()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub griRanges_RowEditEnded(ByVal sender As Object, ByVal e As Telerik.Windows.Controls.GridViewRowEditEndedEventArgs) Handles griRanges.RowEditEnded

        'Quan acabem d'editar els valors actualitzem la BD

        Dim idRange As Integer = 0
        Dim ranMin As String = ""

        Try

            idRange = DirectCast(griRanges.SelectedItem, GG_ranges).RAN_id_range

            ranMin = DirectCast(griRanges.SelectedItem, GG_ranges).RAN_min

            If idRange < Main.lstRanges.Count Then
                ModifyRange(idRange, ranMin)
                UpdateRanges()
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub RadColorPicker_SelectedColorChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)

        'Quan seleccionem un color d'un rang actualitzem la BD

        Dim colorHex As String = ""
        Dim idRange As Integer = 0

        Try

            colorHex = DirectCast(sender, Telerik.Windows.Controls.RadColorPicker).SelectedColor.ToString

            idRange = DirectCast(DirectCast(sender, Telerik.Windows.Controls.RadColorPicker).DataContext, GG_ranges).RAN_id_range

            If idRange < Main.lstRanges.Count Then
                Main.lstRanges.Item(idRange).RAN_color = colorHex
                UpdateRanges()
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Function MountRangesList() As String

        'Amb aquesta funció preparem la llista de rangs abans d'insertar-la a la BD

        Dim strRanges As String = ""

        Try

            Dim tempList = From r In Main.lstRanges Select r Order By r.RAN_min Ascending

            For Each ran In tempList

                strRanges = strRanges & ran.RAN_min & "-" & ran.RAN_max & ";" & ran.RAN_color & "|"

            Next

            strRanges = Mid(strRanges, 1, Len(strRanges) - 1)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return strRanges

    End Function

    Public Sub UpdateRanges()

        'Actualitzem el valor dels rangs en la BD

        Dim strMountedRanges As String

        Try

            strMountedRanges = MountRangesList()

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & strMountedRanges & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='Test' AND CFG_variable='" & cmbTest.Text & "_ranges" & "'")
            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & txtMin.Text & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='Test' AND CFG_variable='" & cmbTest.Text & "_min" & "'")
            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & txtMax.Text & "' WHERE USE_id_user= " & Main.logedUser.USE_id_user & " AND CFG_section='Test' AND CFG_variable='" & cmbTest.Text & "_max" & "'")

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Test ranges for test " & cmbTest.Text & " changed to " & strMountedRanges)
            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Test min for test " & cmbTest.Text & " changed to " & txtMin.Text)
            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Test max for test " & cmbTest.Text & " changed to " & txtMax.Text)

            RefreshUserConfig()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Sub ModifyRange(ByVal ranId As Integer, ByVal ranMin As String)

        Dim lastId As Integer
        Dim rangeMin As Single

        Try

            'Busquem algun rang amb el mateix mínim, si hi és no fem la modificació

            Dim counter = (From r In Main.lstRanges _
                           Where r.RAN_min = ranMin _
                           And r.RAN_id_range <> ranId).Count

            If counter > 0 Then

                Main.ShowMessage(My.Resources.Resource.msg_RangeExists)
                griRanges.Rebind()
                Exit Sub

            End If

            ' Actualitzem el rang actual
            Main.lstRanges.Item(ranId).RAN_min = ranMin

            Dim tempList = From p In Main.lstRanges _
                        Order By p.RAN_min Descending

            lastId = tempList(0).RAN_id_range
            rangeMin = tempList(0).RAN_min

            For Each ran In tempList

                If rangeMin <> 0 Then
                    Main.lstRanges.Item(ran.RAN_id_range).RAN_max = rangeMin
                End If

                rangeMin = ran.RAN_min

            Next

            Select Case cmbTest.Text

                Case "GLU"
                    Main.lstRanges.Item(lastId).RAN_max = 600
                Case "KET"
                    Main.lstRanges.Item(lastId).RAN_max = 12
                Case "HB"
                    Main.lstRanges.Item(lastId).RAN_max = 100
            End Select

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnAddRange_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnAddRange.Click

        'Amb aquest botó es poden afegir rangs (límit de 8)

        Try

            If Main.lstRanges.Count = 8 Then

                Main.ShowMessage(My.Resources.Resource.msg_RangesMax)
                Exit Sub

            End If

            Dim obj_Range As New GG_ranges

            obj_Range.RAN_id_range = Main.lstRanges.Count
            obj_Range.RAN_min = 0
            obj_Range.RAN_max = 0
            obj_Range.RAN_color = "#00000000"

            Main.lstRanges.Add(obj_Range)

            UpdateRanges()
            griRanges.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnRemoveRange_Click(ByVal sender As System.Object, ByVal e As System.Windows.RoutedEventArgs) Handles btnRemoveRange.Click

        'Podem borrar rangs de la llista (n'ha de quedar mínim 1)

        Try

            If Main.lstRanges.Count = 1 Or griRanges.SelectedItem Is Nothing Then

                Exit Sub

            End If

            Dim idRange As Integer = 0

            idRange = DirectCast(griRanges.SelectedItem, GG_ranges).RAN_id_range

            Main.lstRanges.RemoveAt(idRange)

            griRanges.ItemsSource = Main.lstRanges

            Dim counter As Integer = 0

            For Each r In Main.lstRanges

                r.RAN_id_range = counter
                counter = counter + 1

            Next

            UpdateRanges()
            griRanges.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub txtMin_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Input.KeyEventArgs) Handles txtMin.KeyDown

        'Amb aquesta funció ens assegurem que només es poden introduir números en el textbox

        Try

            If (e.Key >= Key.D0 And e.Key <= Key.D9) Or (e.Key >= Key.NumPad0 And e.Key <= Key.NumPad9) Then
                e.Handled = False
            Else
                e.Handled = True
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub txtMax_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Input.KeyEventArgs) Handles txtMax.KeyDown

        'Amb aquesta funció ens assegurem que només es poden introduir números en el textbox

        Try

            If (e.Key >= Key.D0 And e.Key <= Key.D9) Or (e.Key >= Key.NumPad0 And e.Key <= Key.NumPad9) Then
                e.Handled = False
            Else
                e.Handled = True
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub GetAvailableSections()

        Try

            cmbSections.Items.Clear()

            'Carreguem la llista de seccions en el combobox

            For Each obj_section In Main.obj_DAO.GetSections("", "")

                Dim section_item As New SectionItem

                section_item.Visible = False
                section_item.Name = RTrim(obj_section.SEC_description)

                cmbSections.Items.Add(section_item)

            Next

            cmbSections.SelectedIndex = 0

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub griUsers_MouseDoubleClick(ByVal sender As Object, ByVal e As System.Windows.Input.MouseButtonEventArgs) Handles griUsers.MouseDoubleClick

        Try

            isNewUser = False

            'Quan fem doble click sobre un usuari recuperem les seves dades per a editar-les

            Dim obj_user As New GG_users

            obj_user = DirectCast(griUsers.SelectedItem, GG_users)

            txtUserID.Text = obj_user.USE_id_user
            txtName.Text = obj_user.USE_name
            txtLogin.Text = obj_user.USE_login
            txtPassword.Password = obj_user.USE_password
            txtConfirm.Password = obj_user.USE_password

            cmbType.Text = obj_user.USE_type

            If obj_user.USE_active = 0 Then
                chkActive.IsChecked = False
            Else
                chkActive.IsChecked = True
            End If

            'Carreguem els valors de les seccions permeses

            cmbSections.Items.Clear()

            Dim lstSections = Split(Main.GetVariable("User", "Sections"), "|")

            For Each obj_section In Main.obj_DAO.GetSections("", "")

                Dim section_item As New SectionItem

                Dim strSection As String = RTrim(obj_section.SEC_description)

                Dim visible As Boolean = False

                For Each h In lstSections
                    If h = strSection Then
                        visible = True
                    End If
                Next

                If visible = True Then
                    section_item.Visible = True
                Else
                    section_item.Visible = False
                End If

                section_item.Name = RTrim(obj_section.SEC_description)

                If section_item.Name <> "" Then
                    cmbSections.Items.Add(section_item)
                End If

            Next

            cmbSections.SelectedIndex = 0

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Function VerifyErrors() As String

        'Comprovem que no existeixi cap error (camps buits per exemple) abans d'insertar un usuari

        Dim strError As String = ""

        Try

            If txtName.Text = "" Then
                strError = strError & My.Resources.Resource.msg_NameError & vbCrLf
            End If

            If txtLogin.Text = "" Then
                strError = strError & My.Resources.Resource.msg_LoginError & vbCrLf
            End If

            If txtPassword.Password = "" Or txtConfirm.Password = "" Or (txtPassword.Password <> txtConfirm.Password) Then
                strError = strError & My.Resources.Resource.msg_PasswordError & vbCrLf
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return strError

    End Function

    Private Sub ResetUser()

        Try

            isNewUser = True

            'Borrem el contingut dels textbox i establim els valors per defecte

            txtUserID.Text = ""
            txtName.Text = ""
            txtLogin.Text = ""
            txtPassword.Password = ""
            txtConfirm.Password = ""
            cmbType.Text = Main.UserType.ADMIN.ToString
            chkActive.IsChecked = False

            GetAvailableSections()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnResetFields_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles btnResetFields.Click

        Try

            'Cridem la funció de reset dels camps d'usuari

            ResetUser()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnSaveUser_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles btnSaveUser.Click

        'Comprovem que tots els camps tinguin valor i que la contrasenya coincideixi

        Dim strError As String = VerifyErrors()

        Try

            If strError <> "" Then
                Main.ShowMessage(strError)
                Exit Sub
            End If

            Dim USE_active As Long

            If chkActive.IsChecked = True Then
                USE_active = 1
            Else
                USE_active = 0
            End If

            Select Case isNewUser

                Case True

                    Dim USE_id_user As Long

                    USE_id_user = Main.obj_DAO.GetSQL("SELECT TOP 1 USE_id_user FROM GG_users ORDER BY USE_id_user DESC")
                    USE_id_user = USE_id_user + 1

                    txtUserID.Text = USE_id_user

                    Main.obj_DAO.ExecuteSQL("INSERT INTO GG_users (USE_name,USE_login,USE_password,USE_type,USE_active) VALUES('" & txtName.Text & "','" & txtLogin.Text & "','" & txtPassword.Password & "','" & cmbType.Text & "'," & USE_active & ")")

                    For Each variable In Main.obj_DAO.GetConfigurations("GG_configuration.USE_id_user=1", "")

                        Main.obj_DAO.ExecuteSQL("INSERT INTO GG_configuration(USE_id_user,CFG_section,CFG_variable,CFG_value) VALUES(" & USE_id_user & ",'" & variable.CFG_section & "','" & variable.CFG_variable & "','" & variable.CFG_value & "')")

                    Next

                    Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Added new user: " & txtLogin.Text)

                Case False

                    Main.obj_DAO.ExecuteSQL("UPDATE GG_users SET USE_name='" & txtName.Text & "',USE_login='" & txtLogin.Text & "',USE_password='" & txtPassword.Password & "',USE_type='" & cmbType.Text & "',USE_active=" & USE_active & " WHERE USE_id_user=" & txtUserID.Text)

                    Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Updated user: " & txtLogin.Text)

            End Select

            'Gravem la configuració de seccions permeses
            Dim strSections As String = ""

            For Each obj_section In cmbSections.Items

                Dim section_item As New SectionItem

                section_item = DirectCast(obj_section, SectionItem)

                If section_item.Visible = True Then
                    strSections = strSections & section_item.Name & "|"
                End If

            Next

            Main.obj_DAO.ExecuteSQL("UPDATE GG_configuration SET CFG_value='" & strSections & "' WHERE USE_id_user= " & txtUserID.Text & " AND CFG_section='User' AND CFG_variable='Sections'")

            griUsers.ItemsSource = Main.obj_DAO.GetUsers("", "")

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Updated user sections: " & strSections)

            'Si l'usuari s'ha insertat correctament fem un reset dels camps
            ResetUser()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub GetLogs()

        Try

            'Obtenim els logs d'una determinada data

            griLogs.ItemsSource = Main.obj_DAO.GetLogs("LOG_date BETWEEN '" & Format(datFrom.SelectedDate, "dd/MM/yyyy") & " 00:00:00' AND '" & Format(datFrom.SelectedDate, "dd/MM/yyyy") & " 23:59:59'", "")


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub datFrom_DropDownClosed(sender As Object, e As System.Windows.RoutedEventArgs) Handles datFrom.DropDownClosed

        Try

            'Recarreguem els logs al canviar la data

            GetLogs()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub griLogs_DataLoaded(sender As Object, e As System.EventArgs) Handles griLogs.DataLoaded

        Try

            'Calculem el total de logs que hi ha en el grid

            lblTotalLogs.Content = My.Resources.Resource.msg_Total & "= " & griLogs.Items.Count

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

End Class

Class SectionItem

    Private _visible As Boolean
    Public Property Visible() As Boolean
        Get
            Return _visible
        End Get
        Set(ByVal value As Boolean)
            _visible = value
        End Set
    End Property

    Private _name As String
    Public Property Name() As String
        Get
            Return _name
        End Get
        Set(ByVal value As String)
            _name = value
        End Set
    End Property

End Class
