﻿Imports Telerik.Windows.Controls.Charting
Imports Telerik.Windows.Controls
Imports System.Reflection
Imports System.Windows.Input
Imports System.Windows
Imports System.Windows.Media
Imports System.Collections.ObjectModel

Namespace ChartCreator

    Public Class GG_charts

        'Configuració de la gràfica
        Public currentChart As New RadChart
        Public showLegend As Boolean
        Public titleLegend As String
        Public titleChart As String
        Public noDataString As String
        Public showLinesAxisX As Boolean
        Public showLinesAxisY As Boolean
        Public labelRotationAngle As Integer
        Public labelFormat As String
        Public XAutoRange As Boolean
        Public YAutoRange As Boolean
        Public AxisXMinValue As String
        Public AxisXMaxValue As String
        Public AxisYMinValue As String
        Public AxisYMaxValue As String
        Public AxisXStep As Integer
        Public AxisYStep As Integer
        Public axisXIsDate As Boolean
        Public legendPosition As legendPositions
        Public chartType As chartTypes
        Public lstResults As New List(Of IEnumerable(Of Object))

        Public pointSeries As New List(Of Point)
        Public typesSeries As New List(Of serie)
        Public seriesColors As New List(Of System.Windows.Media.Color)
        Public namesSeries As New List(Of String)
        Public namesAxisX As New List(Of String)
        Public tooltipSeries As New List(Of String)
        Public showTooltipSeries As New List(Of Boolean)
        Public categoryAxisX As String
        Public propertyAxisX As String
        Public propertyAxisY As String
        Public lstValues As New List(Of List(Of Double))
        Public lstPointRanges As New List(Of Object)
        Public lastDate As Date
        Public firstDate As Date
        Public chartTheme As New chartThemes
        Public spiderMode As Boolean = False

        Public Enum chartThemes
            Office_Black
            Office_Blue
            Office_Silver
            Summer
            Vista
            Windows7
            Transparent
            Expression_Dark
            Metro
        End Enum

        Public Enum legendPositions
            top
            bottom
            left
            right
        End Enum

        Public Enum chartTypes
            points
            points24h
            bars
            pie
            doughnut
        End Enum

        Public Enum serie
            results
            horizontalRange
            verticalRange
            bars
            stackedBars
            stackedBars100
            pie
            doughnut
            qcout
            qc
        End Enum

        Public Enum point
            circle
            diamond
            hexagon
            square
            roundedSquare
            star5
            star8
            triangle
        End Enum

        Private Function configurarGrafica()

            Try

                If chartType = chartTypes.points Or chartType = chartTypes.points24h Then
                    currentChart.SamplingSettings.SamplingThreshold = 0
                End If

                'Posició de la llegenda en la gràfica
                Select Case legendPosition
                    Case legendPositions.top
                        currentChart.DefaultView.ChartLegendPosition = Controls.Dock.Top
                    Case legendPositions.bottom
                        currentChart.DefaultView.ChartLegendPosition = Controls.Dock.Bottom
                    Case legendPositions.left
                        currentChart.DefaultView.ChartLegendPosition = Controls.Dock.Left
                    Case legendPositions.right
                        currentChart.DefaultView.ChartLegendPosition = Controls.Dock.Right
                    Case Else
                        currentChart.DefaultView.ChartLegendPosition = Controls.Dock.Right
                End Select

                Dim tit As New ChartTitle
                tit.Content = titleChart

                'Apliquem el chartThemes de la gràfica
                Select Case chartTheme
                    Case chartThemes.Office_Black
                        StyleManager.SetTheme(tit, New Office_BlackTheme())
                    Case chartThemes.Office_Blue
                        StyleManager.SetTheme(tit, New Office_BlueTheme())
                    Case chartThemes.Office_Silver
                        StyleManager.SetTheme(tit, New Office_SilverTheme())
                    Case chartThemes.Summer
                        StyleManager.SetTheme(tit, New SummerTheme())
                    Case chartThemes.Vista
                        StyleManager.SetTheme(tit, New VistaTheme())
                    Case chartThemes.Windows7
                        StyleManager.SetTheme(tit, New Windows7Theme())
                    Case chartThemes.Transparent
                        StyleManager.SetTheme(tit, New TransparentTheme())
                    Case chartThemes.Expression_Dark
                        StyleManager.SetTheme(tit, New Expression_DarkTheme())
                    Case chartThemes.Metro
                        StyleManager.SetTheme(tit, New MetroTheme())
                    Case Else
                        StyleManager.SetTheme(tit, New Office_SilverTheme())
                End Select

                currentChart.DefaultView.ChartTitle = tit
                currentChart.DefaultView.ChartTitle.Visibility = Visibility.Visible

                'Configurem el temps de DELAY a l'hora de pintar les gràfiques (per defecte 0)
                currentChart.AnimationSettings.DefaultSeriesDelay = New TimeSpan(0)

                'En funció del paràmetre showLegend mostrem o amaguem la llegenda
                If showLegend = True Then
                    currentChart.DefaultView.ChartLegend.Visibility = Visibility.Visible
                Else
                    currentChart.DefaultView.ChartLegend.Visibility = Visibility.Collapsed
                End If

                'Mostrem el títol de la llegenda passat per paràmetre
                currentChart.DefaultView.ChartLegend.Header = titleLegend

                'Missatge que apareix quan la gràfica no conté resultats
                currentChart.DefaultView.ChartArea.NoDataString = noDataString

                'Amaguem la quadrícula (horitzontal i vertical)
                If showLinesAxisX = True Then
                    currentChart.DefaultView.ChartArea.AxisX.StripLinesVisibility = Visibility.Visible
                Else
                    currentChart.DefaultView.ChartArea.AxisX.StripLinesVisibility = Visibility.Collapsed
                End If

                If showLinesAxisY = True Then
                    currentChart.DefaultView.ChartArea.AxisY.StripLinesVisibility = Visibility.Visible
                Else
                    currentChart.DefaultView.ChartArea.AxisY.StripLinesVisibility = Visibility.Collapsed
                End If

                currentChart.DefaultView.ChartArea.AxisX.LabelRotationAngle = labelRotationAngle
                currentChart.DefaultView.ChartArea.AxisX.DefaultLabelFormat = labelFormat

                currentChart.DefaultView.ChartArea.AxisX.AutoRange = XAutoRange

                If axisXIsDate = True Then

                    If chartType = chartTypes.points24h Then

                        currentChart.DefaultView.ChartArea.AxisX.MinValue = New DateTime(Now.Year, Now.Month, Now.Day, 0, 0, 0).ToOADate()
                        currentChart.DefaultView.ChartArea.AxisX.MaxValue = New DateTime(Now.Year, Now.Month, Now.Day, 0, 0, 0).AddDays(1).ToOADate

                    Else

                        currentChart.DefaultView.ChartArea.AxisX.MinValue = firstDate.AddDays(-1).ToOADate()
                        currentChart.DefaultView.ChartArea.AxisX.MaxValue = lastDate.AddDays(1).ToOADate()

                    End If

                Else
                    currentChart.DefaultView.ChartArea.AxisX.MinValue = AxisXMinValue
                    currentChart.DefaultView.ChartArea.AxisX.MaxValue = AxisXMaxValue
                End If

                If XAutoRange = False Then

                    If chartType = chartTypes.points24h Then

                        currentChart.DefaultView.ChartArea.AxisX.[Step] = 1 / 24
                        currentChart.DefaultView.ChartArea.AxisX.LabelStep = 2
                        currentChart.DefaultView.ChartArea.AxisX.LayoutMode = AxisLayoutMode.Normal

                    Else

                        currentChart.DefaultView.ChartArea.AxisX.Step = AxisXStep

                    End If

                End If

                currentChart.DefaultView.ChartArea.AxisY.AutoRange = YAutoRange

                If YAutoRange = False Then

                    currentChart.DefaultView.ChartArea.AxisY.Step = AxisYStep

                    currentChart.DefaultView.ChartArea.AxisY.MinValue = AxisYMinValue
                    currentChart.DefaultView.ChartArea.AxisY.MaxValue = AxisYMaxValue

                End If

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

            Return currentChart

        End Function

        Private Sub PintarGrafica()

            Try

                If lstResults.Count = 0 Then
                    Exit Sub
                End If

                If lstPointRanges.Count > 0 Then

                    Dim nextIndex As Integer = 0

                    Dim totalSeries As Integer

                    If namesSeries.Count = 1 Then
                        totalSeries = 0
                    Else
                        totalSeries = namesSeries.Count - 1
                    End If

                    For i = 0 To totalSeries

                        Select Case typesSeries(i)

                            Case serie.horizontalRange, serie.qcout

                                DrawRange(seriesColors(i), namesSeries(i), lstValues(i), lstPointRanges(nextIndex), lstPointRanges(nextIndex + 1), propertyAxisX, propertyAxisY, pointSeries(i), typesSeries(i), "H")

                                nextIndex = nextIndex + 2

                            Case serie.verticalRange

                                DrawRange(seriesColors(i), namesSeries(i), lstValues(i), lstPointRanges(nextIndex), lstPointRanges(nextIndex + 1), propertyAxisX, propertyAxisY, pointSeries(i), typesSeries(i), "V")

                                nextIndex = nextIndex + 2

                            Case serie.results, serie.qc

                                Select Case chartType

                                    Case chartTypes.points

                                        'Dibuixem la gràfica de punts
                                        PointsChart(lstResults(i), tooltipSeries(i), showTooltipSeries(i), pointSeries(i), typesSeries(i), namesSeries(i), seriesColors(i))

                                    Case chartTypes.points24h

                                        'Dibuixem la gràfica de punst de 24h
                                        PointsChart(lstResults(i), tooltipSeries(i), showTooltipSeries(i), pointSeries(i), typesSeries(i), namesSeries(i), seriesColors(i))

                                End Select

                        End Select

                    Next i

                End If

                'En cas que sigui un bars / histogramaApilat / histogramaApilat100 / sectors només pintem una serie amb diversos punts

                Select Case typesSeries(0)
                    Case serie.bars, serie.stackedBars, serie.stackedBars100

                        For i = 0 To namesSeries.Count - 1
                            BarsChart(lstResults(i), tooltipSeries(i), showTooltipSeries(i), pointSeries(i), typesSeries(i), namesSeries(i), seriesColors(i))
                        Next

                    Case serie.pie

                        currentChart.PaletteBrushesRepeat = False

                        If lstResults(0).Count = 0 Then
                            Exit Sub
                        End If

                        PieChart(lstResults(0), tooltipSeries(0), showTooltipSeries(0), pointSeries(0), typesSeries(0), namesSeries(0), seriesColors(0))

                    Case serie.doughnut

                        currentChart.PaletteBrushesRepeat = False

                        If lstResults(0).Count = 0 Then
                            Exit Sub
                        End If

                        PieChart(lstResults(0), tooltipSeries(0), showTooltipSeries(0), pointSeries(0), typesSeries(0), namesSeries(0), seriesColors(0))

                End Select

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

        End Sub

        Private Sub DrawRange(ByVal color As System.Windows.Media.Color, ByVal serieName As String, ByVal lstCurrentResults As List(Of Double), ByVal point1 As Object, ByVal point2 As Object, ByVal axisX As String, ByVal axisY As String, ByVal pointStyle As point, ByVal serieType As serie, ByVal rangeDirection As String)

            Dim lstCombined As New ObservableCollection(Of ObservableCollection(Of Object))()
            Dim lstRange As List(Of Object) = New List(Of Object)()

            Dim serieRange As New SeriesMapping()

            Try

                If serieType = serie.qcout Then
                    serieRange = GeneratePointsSerie(serieRange, serieName, New LineSeriesDefinition(), Cursors.Arrow, serieName, True, currentChart.SeriesMappings.Count, axisX, axisY, color, color, color, color, "", False, pointStyle, serieType)
                Else
                    serieRange = GeneratePointsSerie(serieRange, serieName, New LineSeriesDefinition(), Cursors.Arrow, "", False, currentChart.SeriesMappings.Count, axisX, axisY, color, color, color, color, "", False, pointStyle, serieType)
                End If


                If rangeDirection = "H" Then

                    If chartType = GG_charts.chartTypes.points Then

                        Dim pr As PropertyInfo() = point1.GetType.GetProperties()

                        For Each m As PropertyInfo In pr
                            If m.Name = axisX Then
                                m.SetValue(point1, firstDate.AddDays(-1), Nothing)
                            ElseIf m.Name = axisY Then
                                m.SetValue(point1, CInt(lstCurrentResults(0)), Nothing)
                            End If
                        Next

                        lstRange.Add(point1)

                        Dim pr2 As PropertyInfo() = point2.GetType.GetProperties()

                        For Each m As PropertyInfo In pr2
                            If m.Name = axisX Then
                                m.SetValue(point2, lastDate.AddDays(+1), Nothing)
                            ElseIf m.Name = axisY Then
                                m.SetValue(point2, CInt(lstCurrentResults(0)), Nothing)
                            End If
                        Next

                        lstRange.Add(point2)

                    ElseIf chartType = GG_charts.chartTypes.points24h Then

                        Dim pr As PropertyInfo() = point1.GetType.GetProperties()

                        For Each m As PropertyInfo In pr
                            If m.Name = axisX Then
                                m.SetValue(point1, New Date(Now().Year, Now().Month, Now().Day, 0, 0, 0), Nothing)
                            ElseIf m.Name = axisY Then
                                m.SetValue(point1, CInt(lstCurrentResults(0)), Nothing)
                            End If
                        Next

                        lstRange.Add(point1)

                        Dim pr2 As PropertyInfo() = point2.GetType.GetProperties()

                        For Each m As PropertyInfo In pr2
                            If m.Name = axisX Then
                                m.SetValue(point2, New Date(Now().Year, Now().Month, Now().Day, 23, 59, 0), Nothing)
                            ElseIf m.Name = axisY Then
                                m.SetValue(point2, CInt(lstCurrentResults(0)), Nothing)
                            End If
                        Next

                        lstRange.Add(point2)

                    End If

                ElseIf rangeDirection = "V" Then

                    If chartType = GG_charts.chartTypes.points24h Then

                        Dim pr As PropertyInfo() = point1.GetType.GetProperties()

                        For Each m As PropertyInfo In pr
                            If m.Name = axisX Then
                                m.SetValue(point1, New Date(Now().Year, Now().Month, Now().Day, lstCurrentResults(0), 0, 0), Nothing)
                            ElseIf m.Name = axisY Then
                                m.SetValue(point1, CInt(AxisYMinValue), Nothing)
                            End If
                        Next

                        lstRange.Add(point1)

                        Dim pr2 As PropertyInfo() = point2.GetType.GetProperties()

                        For Each m As PropertyInfo In pr2
                            If m.Name = axisX Then
                                m.SetValue(point2, New Date(Now().Year, Now().Month, Now().Day, lstCurrentResults(0), 0, 0), Nothing)
                            ElseIf m.Name = axisY Then
                                m.SetValue(point2, CInt(AxisYMaxValue), Nothing)
                            End If
                        Next

                        lstRange.Add(point2)

                    End If

                End If

                If Not currentChart.ItemsSource Is Nothing Then
                    lstCombined = currentChart.ItemsSource
                End If

                lstCombined.Add(New ObservableCollection(Of Object)(lstRange))

                currentChart.SeriesMappings.Add(serieRange)
                currentChart.ItemsSource = lstCombined

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

        End Sub


        Private Sub PointsChart(ByVal lstCurrentResults As IEnumerable(Of Object), ByVal tooltipFormat As String, ByVal showTooltip As Boolean, ByVal pointStyle As point, ByVal serieType As serie, ByVal name As String, ByVal color As System.Windows.Media.Color)

            Try

                'Declarem i generem les series
                Dim resultsSerie As New SeriesMapping()

                Select Case serieType
                    Case serie.qc
                        resultsSerie = GeneratePointsSerie(resultsSerie, name, New LineSeriesDefinition(), Cursors.Cross, My.Resources.Resource.msg_Result & ": #Y{###.##}", False, currentChart.SeriesMappings.Count, propertyAxisX, propertyAxisY, color, color, color, color, tooltipFormat, showTooltip, pointStyle, serieType)
                    Case Else
                        resultsSerie = GeneratePointsSerie(resultsSerie, name, New ScatterSeriesDefinition(), Cursors.Cross, My.Resources.Resource.msg_Result & ": #Y{###.##}", False, currentChart.SeriesMappings.Count, propertyAxisX, propertyAxisY, color, color, color, color, tooltipFormat, showTooltip, pointStyle, serieType)
                End Select

                'Declarem les llistes amb els resultats
                Dim lstCombined = New ObservableCollection(Of ObservableCollection(Of Object))()

                currentChart.SeriesMappings.Add(resultsSerie)

                If currentChart.ItemsSource Is Nothing Then

                Else
                    lstCombined = currentChart.ItemsSource
                End If

                lstCombined.Add(New ObservableCollection(Of Object)(lstCurrentResults))

                currentChart.ItemsSource = lstCombined

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

        End Sub

        Private Sub BarsChart(ByVal lstCurrentResults As IEnumerable(Of Object), ByVal tooltipFormat As String, ByVal showTooltip As Boolean, ByVal pointStyle As point, ByVal serieType As serie, ByVal name As String, ByVal color As System.Windows.Media.Color)

            Dim resultsSerie As New SeriesMapping()

            Try

                Select Case serieType
                    Case serie.bars
                        currentChart.SeriesMappings.Add(GenerateBarsSerie(resultsSerie, name, New BarSeriesDefinition(), Cursors.Cross, labelFormat, False, currentChart.SeriesMappings.Count, categoryAxisX, propertyAxisY, color, color, color, color, tooltipFormat, showTooltip, pointStyle, serieType))
                    Case serie.stackedBars
                        currentChart.SeriesMappings.Add(GenerateBarsSerie(resultsSerie, name, New StackedBarSeriesDefinition(), Cursors.Cross, labelFormat, False, currentChart.SeriesMappings.Count, categoryAxisX, propertyAxisY, color, color, color, color, tooltipFormat, showTooltip, pointStyle, serieType))
                    Case serie.stackedBars100
                        currentChart.SeriesMappings.Add(GenerateBarsSerie(resultsSerie, name, New StackedBar100SeriesDefinition(), Cursors.Cross, labelFormat, False, currentChart.SeriesMappings.Count, categoryAxisX, propertyAxisY, color, color, color, color, tooltipFormat, showTooltip, pointStyle, serieType))
                End Select

                If seriesColors(0) <> Nothing Then
                    currentChart.DefaultView.ChartArea.PaletteBrushes.Clear()
                    For Each color In seriesColors
                        currentChart.DefaultView.ChartArea.PaletteBrushes.Add(New SolidColorBrush(color))
                    Next
                End If

                currentChart.ItemsSource = lstResults

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

        End Sub

        Private Sub PieChart(ByVal lstCurrentResults As IEnumerable(Of Object), ByVal tooltipFormat As String, ByVal showTooltip As Boolean, ByVal pointStyle As point, ByVal serieType As serie, ByVal name As String, ByVal color As System.Windows.Media.Color)

            Dim resultsSerie As New SeriesMapping()

            Try

                Select Case serieType
                    Case serie.pie
                        currentChart.SeriesMappings.Add(GeneratePieSerie(resultsSerie, name, New PieSeriesDefinition(), Cursors.Cross, labelFormat, False, currentChart.SeriesMappings.Count, categoryAxisX, propertyAxisY, tooltipFormat, showTooltip, pointStyle, serieType))
                    Case serie.doughnut
                        currentChart.SeriesMappings.Add(GeneratePieSerie(resultsSerie, name, New DoughnutSeriesDefinition(), Cursors.Cross, labelFormat, False, currentChart.SeriesMappings.Count, categoryAxisX, propertyAxisY, tooltipFormat, showTooltip, pointStyle, serieType))
                End Select

                currentChart.ItemsSource = lstCurrentResults

                If seriesColors(0) <> Nothing Then
                    currentChart.DefaultView.ChartArea.PaletteBrushes.Clear()
                    For Each color In seriesColors
                        currentChart.DefaultView.ChartArea.PaletteBrushes.Add(New SolidColorBrush(color))
                    Next
                End If

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

        End Sub

        Private Function GeneratePointsSerie(ByVal serie As SeriesMapping, ByVal legendLabel As String, ByVal serieType As Telerik.Windows.Controls.Charting.ISeriesDefinition, ByVal cursorType As System.Windows.Input.Cursor, ByVal itemLabelFormat As String, ByVal showItemLabels As Boolean, ByVal collectionIndex As Integer, ByVal X As String, ByVal Y As String, ByVal pointColor As System.Windows.Media.Color, ByVal pointBorderColor As System.Windows.Media.Color, ByVal lineColor As System.Windows.Media.Color, ByVal pointLabelColor As System.Windows.Media.Color, ByVal tooltipFormat As String, ByVal showTooltip As Boolean, ByVal pointStyle As point, ByVal pointType As serie)

            Try

                Dim style2 As New Style

                style2.Setters.Add(New Setter(PointMark.SizeProperty, 2.0))

                Dim style5 As New Style

                style5.Setters.Add(New Setter(PointMark.SizeProperty, 5.0))

                Dim style10 As New Style

                style10.Setters.Add(New Setter(PointMark.SizeProperty, 10.0))

                'Definim el tipus, la pointStyle del cursor, tipus de punt, format del label del punt
                serie.LegendLabel = legendLabel
                serie.SeriesDefinition = serieType
                serie.SeriesDefinition.Appearance.Cursor = cursorType

                Select Case pointType

                    Case GG_charts.serie.horizontalRange, GG_charts.serie.verticalRange
                        serie.SeriesDefinition.PointMarkItemStyle = style2
                    Case GG_charts.serie.results, GG_charts.serie.qc
                        serie.SeriesDefinition.PointMarkItemStyle = style5
                    Case GG_charts.serie.qcout
                        serie.SeriesDefinition.PointMarkItemStyle = style2
                        serie.SeriesDefinition.Appearance.StrokeThickness = 80
                End Select

                serie.SeriesDefinition.ItemLabelFormat = itemLabelFormat
                serie.SeriesDefinition.ShowItemLabels = showItemLabels

                Select Case pointStyle

                    Case point.circle
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.Circle
                    Case point.square
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.Square
                    Case point.roundedSquare
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.SquareRounded
                    Case point.diamond
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.Diamond
                    Case point.star5
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.StarFiveRay
                    Case point.star8
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.StarEightRay
                    Case point.hexagon
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.Hexagon
                    Case point.triangle
                        serie.SeriesDefinition.Appearance.PointMark.Shape = MarkerShape.Triangle

                End Select

                serie.SeriesDefinition.ShowItemToolTips = showTooltip
                serie.SeriesDefinition.ItemToolTipFormat = tooltipFormat

                'Indiquem quina propietat és la X i quina és la Y
                serie.CollectionIndex = collectionIndex
                serie.ItemMappings.Add(New ItemMapping(X, DataPointMember.XValue))
                serie.ItemMappings.Add(New ItemMapping(Y, DataPointMember.YValue))

                'Assignem el color als punts i series

                serie.SeriesDefinition.Appearance.PointMark.Fill = New SolidColorBrush(pointColor)
                serie.SeriesDefinition.Appearance.PointMark.Stroke = New SolidColorBrush(pointBorderColor)
                serie.SeriesDefinition.Appearance.Stroke = New SolidColorBrush(lineColor)
                serie.SeriesDefinition.Appearance.Fill = New SolidColorBrush(pointLabelColor)

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

            Return serie

        End Function

        Private Function GenerateBarsSerie(ByVal serie As SeriesMapping, ByVal legendLabel As String, ByVal serieType As Telerik.Windows.Controls.Charting.ISeriesDefinition, ByVal cursorType As System.Windows.Input.Cursor, ByVal itemLabelFormat As String, ByVal showItemLabels As Boolean, ByVal collectionIndex As Integer, ByVal X As String, ByVal Y As String, ByVal pointColor As System.Windows.Media.Color, ByVal pointBorderColor As System.Windows.Media.Color, ByVal lineColor As System.Windows.Media.Color, ByVal pointLabelColor As System.Windows.Media.Color, ByVal tooltipFormat As String, ByVal showTooltip As Boolean, ByVal pointStyle As point, ByVal pointType As serie)


            Try

                'Definim el tipus, pointStyle, cursor, tipus de punt, format del label del punt...
                serie.LegendLabel = legendLabel

                serie.CollectionIndex = collectionIndex
                serie.SeriesDefinition = serieType

                serie.SeriesDefinition.ShowItemToolTips = showTooltip
                serie.SeriesDefinition.ItemToolTipFormat = tooltipFormat

                serie.SeriesDefinition.ItemLabelFormat = itemLabelFormat

                'Indiquem quina propietat és la X i quina la Y
                serie.ItemMappings.Add(New ItemMapping(X, DataPointMember.XCategory))
                serie.ItemMappings.Add(New ItemMapping(Y, DataPointMember.YValue))

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

            Return serie

        End Function

        Private Function GeneratePieSerie(ByVal serie As SeriesMapping, ByVal legendLabel As String, ByVal serieType As Telerik.Windows.Controls.Charting.ISeriesDefinition, ByVal cursorType As System.Windows.Input.Cursor, ByVal itemLabelFormat As String, ByVal showItemLabels As Boolean, ByVal collectionIndex As Integer, ByVal X As String, ByVal Y As String, ByVal tooltipFormat As String, ByVal showTooltip As Boolean, ByVal pointStyle As point, ByVal pointType As serie)

            Try

                'Definim el tipus, pointStyle del cursor, tipus de punt, format del label del punt...
                serie.LegendLabel = legendLabel

                If InStr(serieType.GetType.ToString, "Doughnut") > 0 Then

                    If spiderMode = True Then
                        DirectCast(serieType, DoughnutSeriesDefinition).RadiusFactor = 0.5
                        DirectCast(serieType, DoughnutSeriesDefinition).LabelSettings.ShowConnectors = True
                        DirectCast(serieType, DoughnutSeriesDefinition).LabelSettings.SpiderModeEnabled = True
                    End If

                End If

                If InStr(serieType.GetType.ToString, "Pie") > 0 Then

                    If spiderMode = True Then
                        DirectCast(serieType, PieSeriesDefinition).RadiusFactor = 0.5
                        DirectCast(serieType, PieSeriesDefinition).LabelSettings.ShowConnectors = True
                        DirectCast(serieType, PieSeriesDefinition).LabelSettings.SpiderModeEnabled = True
                    End If

                End If

                collectionIndex = collectionIndex + 1

                serie.SeriesDefinition = serieType

                serie.SeriesDefinition.ShowItemToolTips = showTooltip
                serie.SeriesDefinition.ItemToolTipFormat = tooltipFormat

                serie.SeriesDefinition.ItemLabelFormat = itemLabelFormat

                'Indiquem quina propietat és la X i quina la Y
                serie.ItemMappings.Add(New ItemMapping(X, DataPointMember.XCategory))
                serie.ItemMappings.Add(New ItemMapping(Y, DataPointMember.YValue))

            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

            Return serie

        End Function

        Public Function DrawChart() As RadChart

            'Configurem la gràfica amb els valors adequats i després la retornem

            currentChart.SeriesMappings.Clear()
            currentChart.ItemsSource = Nothing

            configurarGrafica()

            PintarGrafica()

            Return currentChart

        End Function

        Public Shared Function CalculateStep(ByVal dateMin As Date, ByVal dateMax As Date) As Long

            Dim myStep As Long

            Try

                myStep = DateDiff(DateInterval.Day, dateMin.AddDays(-1), dateMax.AddDays(+1))

                If myStep < 6 Then
                    myStep = myStep + 1
                Else
                    myStep = myStep / 4
                End If


            Catch ex As Exception

                Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

            End Try

            Return myStep

        End Function

    End Class

End Namespace
