﻿Imports Telerik.Windows.Controls
Imports Telerik.Windows.Controls.TransitionEffects
Imports GlucoGuardian.Business.Logic.Objects
Imports System.Collections.ObjectModel
Imports System.Configuration

Class Main

    Public Shared mainMenuTransition As New RadTransitionControl
    Public Shared mainContentTransition As New RadTransitionControl

    Public Shared lstPeriods As New ObservableCollection(Of GG_periods)
    Public Shared lstRanges As New ObservableCollection(Of GG_ranges)

    Public Shared logedUser As New GG_users
    Public Shared userConfig As New List(Of GG_configuration)
    Public Shared obj_DAO As New GlucoGuardian_DAO.DAO

    Public Enum UserType

        ADMIN
        ENDOCRINE
        LAB
        MANAGER

    End Enum

    Sub New()

        Try

            InitializeComponent()

            'Apliquem un estil a tota l'aplicació
            StyleManager.ApplicationTheme = New Telerik.Windows.Controls.Office_SilverTheme

            'Traduïm l'aplicació a l'idioma escollit
            My.Resources.Resource.Culture = New System.Globalization.CultureInfo(ConfigurationManager.AppSettings("Language"))

            'Inicialitzem l'objecte WCF amb els paràmetres adequats
            obj_DAO.SetWebService(ConfigurationManager.AppSettings("WCFLocation"))

        Catch ex As Exception

            ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub Main_Loaded(ByVal sender As Object, ByVal e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        logedUser.USE_id_user = 1

        Try

            Dim assembly As System.Reflection.Assembly = System.Reflection.Assembly.GetExecutingAssembly()
            Dim version As System.Version = assembly.GetName().Version

            Me.Title = "Gluco Guardian - " & version.Major & "." & version.Minor & "." & version.Build

            mainContentTransition.Duration = New TimeSpan(0, 0, 1)
            mainContentTransition.Transition = New SlideAndZoomTransition

            canMainMenu.Children.Add(mainMenuTransition)
            canMainContent.Children.Add(mainContentTransition)

            ChangeMainMenu("Login.xaml")
            ChangeContent("Home.xaml")

        Catch ex As Exception

            ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Shared Sub ChangeMainMenu(ByVal xaml As String)

        'Apliquem el nou xaml al menu principal

        Dim uri As Uri

        Try

            uri = New Uri(xaml, UriKind.Relative)

            Dim frameContenido As Frame = New Frame
            frameContenido.Source = uri

            frameContenido.Width = mainMenuTransition.Width
            frameContenido.Height = mainMenuTransition.Height

            mainMenuTransition.Content = frameContenido

        Catch ex As Exception

            ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Shared Sub ChangeContent(ByVal xaml As String)

        'Apliquem un nou xaml al contingut principal

        Dim uri As Uri

        Try

            uri = New Uri(xaml, UriKind.Relative)

            Dim frameContenido As Frame = New Frame
            frameContenido.Source = uri

            frameContenido.Width = mainContentTransition.Width
            frameContenido.Height = mainContentTransition.Height

            mainContentTransition.Content = frameContenido

        Catch ex As Exception

            ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Shared Sub ShowMessage(ByVal strMessage As String)

        'Mostra un missatge passat per paràmetre

        Try

            Dim msgWindow As New RadWindow
            msgWindow.Alert(strMessage)

        Catch ex As Exception

            ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Shared Function GetVariable(ByVal strSection As String, ByVal strVariable As String) As String

        Dim strValue As String = ""

        Try

            'Obtenim el valor d'una secció / variable

            strValue = (From cfg In userConfig Where cfg.CFG_section = strSection And cfg.CFG_variable = strVariable Select cfg)(0).CFG_value

        Catch ex As Exception

            ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return strValue

    End Function

    Private Sub Main_Unloaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Unloaded

        End

    End Sub

    Public Shared Sub ExceptionManagement(ByVal strMethod As String, ByVal exc As Exception)

        'Tractem l'excepció passada per paràmetre

        Console.WriteLine(strMethod & ": " & exc.Message)
        ShowMessage(strMethod & ": " & exc.Message)
        SaveLog(strMethod, exc.Message)

    End Sub

    Public Shared Sub SaveLog(ByVal strMethod As String, ByVal strMessage As String)

        'Insertem el log en la BD

        obj_DAO.ExecuteSQL("INSERT INTO GG_logs (USE_id_user,LOG_type,LOG_method,LOG_computer,LOG_description,LOG_date) VALUES(" & logedUser.USE_id_user & ",'Gluco Guardian','" & strMethod & "','" & Environment.MachineName.ToString & "','" & Replace(strMessage, "'", "´") & "','" & Format(Now, "yyyyMMdd HH:mm:ss") & "')")

    End Sub

End Class
