﻿Imports GlucoGuardian.Business.Logic.Objects
Imports System.Collections.ObjectModel
Imports Telerik.Windows.Controls.Charting
Imports System.ComponentModel
Imports Telerik.Windows.Data
Imports Telerik.Windows.Controls

Class Patients

    Dim lstResults As ICollectionView

    Private Sub Patients_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        Try

            'Carreguem el llistat de pacients en el Carrousel
            carPatients.ItemsSource = Main.obj_DAO.GetPatients("", "")
            carPatients.SelectedItem = carPatients.Items(0)

            'Obtenim els resultats del pacient seleccionat
            GetPatientInfo()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub carPatients_SelectionChanged(sender As Object, e As Telerik.Windows.Controls.SelectionChangeEventArgs) Handles carPatients.SelectionChanged

        Try

            'Al canviar el pacient seleccionat recarreguem els seus resultats
            GetPatientInfo()


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub GetResultColor(sender As System.Object, e As System.Windows.RoutedEventArgs)

        Dim objResult As New GG_results

        Try

            objResult = DirectCast(sender.Parent.Parent.Datacontext, GG_results)

            'Segons el rang de normalitat acceptat en el mesurador marquem el resultat com a BAIX, NORMAL o ALT

            If objResult.RES_result = 0 And objResult.RES_incidence <> "" Then
                sender.Foreground = Brushes.Black
            ElseIf objResult.RES_result < objResult.RES_low Then
                sender.Foreground = New SolidColorBrush(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "LowColor")))
            ElseIf objResult.RES_result >= objResult.RES_low And objResult.RES_result <= objResult.RES_high Then
                sender.Foreground = New SolidColorBrush(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "NormalColor")))
            ElseIf objResult.RES_result > objResult.RES_high Then
                sender.Foreground = New SolidColorBrush(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub GetPatientInfo()

        Try

            Dim objPatient As New GG_patients
            objPatient = DirectCast(carPatients.SelectedItem, GG_patients)

            'Obtenim les dades de glucosa del pacient actual
            lstResults = New QueryableCollectionView(New ObservableCollection(Of GG_results)(Main.obj_DAO.GetResults("GG_tests.TES_id_test=1 AND RES_type='RES' AND GG_patients.PAT_id_patient=" & objPatient.PAT_id_patient, "RES_date DESC")))
            lblPatient.Content = objPatient.PAT_surname & ", " & objPatient.PAT_name & "  [" & objPatient.PAT_history_number & "]"

            griResults.ItemsSource = lstResults
            frmResults.ItemsSource = lstResults
            griResults.Rebind()

            'Dibuixem la gràfica dels resultats del pacient
            DrawPatientResults()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub DrawPatientResults()

        Try

            Dim obj_chart = New ChartCreator.GG_charts

            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver
            obj_chart.titleChart = lblPatient.Content

            'Si la gràfica no conté resultats mostrem un missatge explicatiu
            If lstResults.IsEmpty = True Then

                Dim title As New ChartTitle
                title.Content = lblPatient.Content

                chaPatientResults.DefaultView.ChartTitle = title
                chaPatientResults.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
                chaPatientResults.ItemsSource = Nothing

                Exit Sub

            End If

            Dim lastDate = (From r In lstResults _
            Select r.RES_date).Max

            Dim firstDate = (From r In lstResults _
            Select r.RES_date).Min

            'Indiquem si l'eix de les X és del tipus DATE
            obj_chart.axisXIsDate = True

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = False
            obj_chart.YAutoRange = False

            'Indiquem els valors mínim i màxim de l'eix de les X (Si axisXIsDate és True es deixa en "")
            obj_chart.AxisXMaxValue = ""
            obj_chart.AxisXMinValue = ""

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = "600"
            obj_chart.AxisYMinValue = "0"

            'Indiquem els Step dels eixos (Si axisXIsDate és True no es tenen en compte)
            obj_chart.AxisXStep = ChartCreator.GG_charts.CalculateStep(firstDate, lastDate)
            obj_chart.AxisYStep = 100

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = firstDate.AddDays(-1)
            obj_chart.lastDate = lastDate.AddDays(+1)

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = chaPatientResults

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "dd/MM/yy"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 0

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = False

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.bottom

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.propertyAxisX = "RES_date"
            obj_chart.propertyAxisY = "RES_result"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.points

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = ""
            obj_chart.titleChart = lblPatient.Content


            '*******************Configuració de les series******************************
            Dim valores = New List(Of Double)

            'Indiquem la 1ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "LowColor")))
            valores.Add(CDbl(Main.GetVariable("Test", "GLU_min")))
            obj_chart.lstValues.Add(valores)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            valores = New List(Of Double)

            'Indiquem la 2ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            valores.Add(CDbl(Main.GetVariable("Test", "GLU_max")))
            obj_chart.lstValues.Add(valores)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            valores = New List(Of Double)

            'Indiquem la 3ª serie (RESULTATS)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.results)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.circle)
            obj_chart.seriesColors.Add(System.Windows.Media.Colors.Black)
            valores.Add(0)
            obj_chart.lstResults.Add(lstResults.Cast(Of Object)())
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(True)

            Dim resultsTooltip As String
            resultsTooltip = My.Resources.Resource.msg_Date & ": #RES_date{dd/MM/yyyy}" & vbCrLf & _
                                My.Resources.Resource.msg_Hour & ": #RES_hour{HH:mm:ss}" & vbCrLf & _
                                My.Resources.Resource.msg_Result & ": #Y{##0.##} " & Main.obj_DAO.GetTests("TES_id_test=1", "")(0).UNI_description & vbCrLf & _
                                My.Resources.Resource.msg_Patient & ": #PAT_history_number" & vbCrLf & _
                                My.Resources.Resource.msg_Service & ": #SER_description"

            obj_chart.tooltipSeries.Add(resultsTooltip)

            chaPatientResults = obj_chart.DrawChart()
            chaPatientResults.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbSections_DropDownClosed(sender As System.Object, e As System.EventArgs)

        'Quan seleccionem un valor del combobox modifiquem l'objecte associat

        Try

            If DirectCast(sender, RadComboBox).SelectedItem Is Nothing Then
                Exit Sub
            End If

            DirectCast(frmResults.CurrentItem, GG_results).SEC_id_section = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_sections).SEC_id_section
            DirectCast(frmResults.CurrentItem, GG_results).SEC_description = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_sections).SEC_description

            griResults.Rebind()


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbSections_Loaded(sender As System.Object, e As System.Windows.RoutedEventArgs)

        'Inicialitzem la llista de Seccions

        Try

            Dim obj_combo As New RadComboBox
            obj_combo = DirectCast(sender, RadComboBox)

            obj_combo.DisplayMemberPath = "SEC_description"
            obj_combo.ItemsSource = Main.obj_DAO.GetSections("", "")

            obj_combo.Text = DirectCast(frmResults.CurrentItem, GG_results).SEC_description
            obj_combo.SelectedItem = griResults.SelectedItem

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbServices_DropDownClosed(sender As System.Object, e As System.EventArgs)

        'Quan seleccionem un valor del combobox modifiquem l'objecte associat

        Try

            If DirectCast(sender, RadComboBox).SelectedItem Is Nothing Then
                Exit Sub
            End If

            DirectCast(frmResults.CurrentItem, GG_results).SER_id_service = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_services).SER_id_service
            DirectCast(frmResults.CurrentItem, GG_results).SER_description = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_services).SER_description

            griResults.Rebind()


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbServices_Loaded(sender As System.Object, e As System.Windows.RoutedEventArgs)

        'Inicialitzem la llista de Serveis

        Try

            Dim obj_combo As New RadComboBox
            obj_combo = DirectCast(sender, RadComboBox)

            obj_combo.DisplayMemberPath = "SER_description"
            obj_combo.ItemsSource = Main.obj_DAO.GetServices("", "")

            obj_combo.Text = DirectCast(frmResults.CurrentItem, GG_results).SER_description
            obj_combo.SelectedItem = griResults.SelectedItem

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbPatients_DropDownClosed(sender As System.Object, e As System.EventArgs)

        'Quan seleccionem un valor del combobox modifiquem l'objecte associat

        Try

            If DirectCast(sender, RadComboBox).SelectedItem Is Nothing Then
                Exit Sub
            End If

            DirectCast(frmResults.CurrentItem, GG_results).PAT_id_patient = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_patients).PAT_id_patient
            DirectCast(frmResults.CurrentItem, GG_results).PAT_history_number = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_patients).PAT_history_number

            griResults.Rebind()


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbPatients_Loaded(sender As System.Object, e As System.Windows.RoutedEventArgs)

        'Inicialitzem la llista de Pacients

        Try

            Dim obj_combo As New RadComboBox
            obj_combo = DirectCast(sender, RadComboBox)

            obj_combo.DisplayMemberPath = "PAT_history_number"
            obj_combo.ItemsSource = Main.obj_DAO.GetPatients("", "")

            obj_combo.Text = DirectCast(frmResults.CurrentItem, GG_results).PAT_history_number
            obj_combo.SelectedItem = griResults.SelectedItem

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbAnalyzers_DropDownClosed(sender As System.Object, e As System.EventArgs)

        'Quan seleccionem un valor del combobox modifiquem l'objecte associat

        Try

            If DirectCast(sender, RadComboBox).SelectedItem Is Nothing Then
                Exit Sub
            End If

            DirectCast(frmResults.CurrentItem, GG_results).ANA_id_analyzer = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_analyzers).ANA_id_analyzer
            DirectCast(frmResults.CurrentItem, GG_results).ANA_factory_number = DirectCast(DirectCast(sender, RadComboBox).SelectedItem, GG_analyzers).ANA_factory_number

            griResults.Rebind()


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbAnalyzers_Loaded(sender As System.Object, e As System.Windows.RoutedEventArgs)

        'Inicialitzem la llista d'Analitzadors

        Try

            Dim obj_combo As New RadComboBox
            obj_combo = DirectCast(sender, RadComboBox)

            obj_combo.DisplayMemberPath = "ANA_factory_number"
            obj_combo.ItemsSource = Main.obj_DAO.GetAnalyzers("", "")

            obj_combo.Text = DirectCast(frmResults.CurrentItem, GG_results).ANA_factory_number
            obj_combo.SelectedItem = griResults.SelectedItem

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub frmResults_EditEnded(sender As Object, e As Telerik.Windows.Controls.Data.DataForm.EditEndedEventArgs) Handles frmResults.EditEnded

        Try

            'Quan acaba l'edició d'un resultat comprovem si s'ha fet clic en Acceptar o Cancel·lar

            If e.EditAction = Telerik.Windows.Controls.Data.DataForm.EditAction.Commit Then
                SaveResult()
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub SaveResult()

        Try

            'Recuperem l'objecte actual

            If frmResults.CurrentItem Is Nothing Then
                Exit Sub
            End If

            Dim result As New GG_results
            result = DirectCast(frmResults.CurrentItem, GG_results)

            'Comprovem si existeix a la BD

            If result.RES_id_result = 0 Then

                'Si no existeix l'objecte fem l'INSERT 

                result.RES_low = CInt(Main.GetVariable("Test", "GLU_min"))
                result.RES_high = CInt(Main.GetVariable("Test", "GLU_max"))

                Main.obj_DAO.ExecuteSQL("INSERT INTO GG_results (RES_barcode,PAT_id_patient,RES_date,SEC_id_section,SER_id_service,ANA_id_analyzer,RES_result,TES_id_test,RES_high,RES_low,RES_type,RES_operator) VALUES('MANUAL'," & result.PAT_id_patient & ",'" & Format(Now, "yyyyMMdd HH:mm:ss") & "'," & result.SEC_id_section & "," & result.SER_id_service & "," & result.ANA_id_analyzer & "," & result.RES_result & ",1," & result.RES_high & "," & result.RES_low & ",'RES','" & Main.logedUser.USE_login & "')")

                Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Inserted result, patient: " & result.PAT_history_number & ", value: " & result.RES_result)

            Else

                'Si existeix l'objecte fem un UPDATE

                Main.obj_DAO.ExecuteSQL("UPDATE GG_results SET SEC_id_section=" & result.SEC_id_section & ",SER_id_service=" & result.SER_id_service & ",PAT_id_patient=" & result.PAT_id_patient & ",RES_result=" & result.RES_result & ",ANA_id_analyzer=" & result.ANA_id_analyzer & " WHERE RES_id_result=" & result.RES_id_result)

                Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Updated result, patient: " & result.PAT_history_number & ", RES_id_result: " & result.RES_id_result)

            End If

            griResults.Rebind()

            GetPatientInfo()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub frmResults_DeletingItem(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles frmResults.DeletingItem

        Try

            e.Cancel = True

            If MsgBox(My.Resources.Resource.msg_AreYouSure, MsgBoxStyle.OkCancel) = MsgBoxResult.Ok Then

                'Recuperem l'objecte eliminat
                Dim result As New GG_results
                result = DirectCast(frmResults.CurrentItem, GG_results)

                'Eliminem l'objecte de la BD

                griResults.Items.Remove(result)

                Main.obj_DAO.ExecuteSQL("DELETE FROM GG_results WHERE RES_id_result" & result.RES_id_result)

                Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Deleted result, RES_id_result: " & result.RES_id_result)

            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

End Class

