﻿Imports GlucoGuardian.Business.Logic.Objects
Imports System.Math
Imports Telerik.Windows.Data
Imports System.Collections.ObjectModel
Imports Telerik.Windows.Controls.Charting
Imports System.IO

Public Class Results

    Dim lstResults As New ObservableCollection(Of GG_results)

    Dim currentType As String = "RES"
    Dim currentTest As Integer = 1
    Dim currentChart As String = "ChartGlobal"

    Dim avgGrid As Double
    Dim CVGrid As Double
    Dim SDGrid As Double

    Dim SEC_id_section As Integer = 0

    Private Sub Results_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded

        Try

            VerifyUserType()

            'Inicialitzem la data d'avui
            datTo.SelectedDate = Now

            'Calculem la data des d'on volem obtenir resultats
            datFrom.SelectedDate = DateAdd(DateInterval.Day, -CInt(Main.GetVariable("General", "ShowLast")), Now)

            'Inicialitzem les proves que volem mostrar en resultats
            cmbResultsTests.ItemsSource = Main.obj_DAO.GetTests("", "")
            cmbResultsTests.SelectedIndex = 0

            'Inicialitzem les proves que volem mostrar en QC
            cmbQCTests.ItemsSource = Main.obj_DAO.GetTests("", "")
            cmbQCTests.SelectedIndex = 0

            'Inicialitzem les seccions permeses de la configuració de l'Usuari
            InitializeSections()

            InitializeResults(SEC_id_section)

            InitializeRanges(Main.GetVariable("Test", "GLU_ranges"))

            ApplyFilter()

            'Inicialitzem els valors dels combos per filtrar segons els resultats del grid
            InitializeCombos()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub VerifyUserType()

        'Segons el tipus d'usuari li permetem veure unes pestanyes o altres

        Try

            Select Case Main.logedUser.USE_type

                Case Main.UserType.ADMIN.ToString

                    'L'administrador no pot accedir a aquesta pantalla

                Case Main.UserType.ENDOCRINE.ToString

                    'L'endocrí té accés als resultats però no al control de qualitat ni els errors

                    tabResults.Visibility = Windows.Visibility.Visible
                    tabGlobal.Visibility = Windows.Visibility.Visible
                    tabService.Visibility = Windows.Visibility.Visible
                    tab24h.Visibility = Windows.Visibility.Visible
                    tabLineal.Visibility = Windows.Visibility.Visible

                    tabQC.Visibility = Windows.Visibility.Collapsed
                    tabQCRanges.Visibility = Windows.Visibility.Collapsed
                    TabLJ.Visibility = Windows.Visibility.Collapsed

                    tabErrors.Visibility = Windows.Visibility.Collapsed

                    currentType = "RES"
                    currentChart = "ChartGlobal"
                    tabGlobal.IsSelected = True

                Case Main.UserType.LAB.ToString

                    'El responsable del laboratori té accés al control de qualitat però no als resultats ni errors

                    tabResults.Visibility = Windows.Visibility.Collapsed
                    tabGlobal.Visibility = Windows.Visibility.Collapsed
                    tabService.Visibility = Windows.Visibility.Collapsed
                    tab24h.Visibility = Windows.Visibility.Collapsed
                    tabLineal.Visibility = Windows.Visibility.Collapsed

                    tabQC.Visibility = Windows.Visibility.Visible
                    tabQCRanges.Visibility = Windows.Visibility.Visible
                    TabLJ.Visibility = Windows.Visibility.Visible

                    tabErrors.Visibility = Windows.Visibility.Collapsed

                    currentType = "QC"
                    currentChart = "ChartQCRanges"
                    tabQCRanges.IsSelected = True

                Case Main.UserType.MANAGER.ToString

                    'El gerent de l'hospital té accés únicament als errors

                    tabResults.Visibility = Windows.Visibility.Collapsed
                    tabGlobal.Visibility = Windows.Visibility.Visible
                    tabService.Visibility = Windows.Visibility.Visible
                    tab24h.Visibility = Windows.Visibility.Collapsed
                    tabLineal.Visibility = Windows.Visibility.Collapsed

                    tabQC.Visibility = Windows.Visibility.Collapsed
                    tabQCRanges.Visibility = Windows.Visibility.Collapsed
                    TabLJ.Visibility = Windows.Visibility.Collapsed

                    tabErrors.Visibility = Windows.Visibility.Visible

                    tabErrors.Focus()

            End Select


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub InitializeSections()

        Try

            'Inicialitzem el combobox de les seccions
            Dim lstSections = Split(Main.GetVariable("User", "Sections"), "|")

            For Each sec In Main.obj_DAO.GetSections("", "")
                If (From section In lstSections Where section = sec.SEC_description).Count <> 0 Then
                    cmbSections.Items.Add(sec)
                End If
            Next

            cmbSections.DisplayMemberPath = "SEC_description"

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub


    Private Sub InitializeResults(ByVal SEC_id_section As Integer)

        Try

            'Apliquem les condicions de data i hora

            Dim strCondition As String = ""

            If datFrom.SelectedDate.ToString <> Nothing And datTo.SelectedDate.ToString = Nothing Then
                strCondition = "RES_type='" & currentType & "' AND RES_date >= '" & Format(datFrom.SelectedDate, "dd/MM/yyyy") & " 00:00:00'"
            End If

            If datFrom.SelectedDate.ToString = Nothing And datTo.SelectedDate.ToString <> Nothing Then
                strCondition = "RES_type='" & currentType & "' AND RES_date <= '" & Format(datTo.SelectedDate, "dd/MM/yyyy") & " 23:59:59'"
            End If

            If datFrom.SelectedDate.ToString <> Nothing And datTo.SelectedDate.ToString <> Nothing Then
                strCondition = "RES_type='" & currentType & "' AND RES_date BETWEEN '" & Format(datFrom.SelectedDate, "dd/MM/yyyy") & " 00:00:00' AND '" & Format(datTo.SelectedDate, "dd/MM/yyyy") & " 23:59:59'"
            End If

            If datFrom.SelectedDate.ToString = Nothing And datTo.SelectedDate.ToString = Nothing Then
                strCondition = "RES_type='" & currentType & "'"
            End If

            'Si passem un hospital com a paràmetre l'adjuntem com a condició

            If SEC_id_section <> 0 Then

                strCondition = strCondition & " AND " & " GG_sections.SEC_id_section=" & SEC_id_section

            Else

                strCondition = strCondition & " AND ("

                For Each sec In cmbSections.Items

                    If sec.SEC_description <> "" Then
                        strCondition = strCondition & " GG_sections.SEC_id_section=" & sec.SEC_id_section & " OR "
                    End If

                Next

            End If

            If Right(strCondition, 5) = "AND (" Then
                strCondition = Mid(strCondition, 1, Len(strCondition) - 5)
            End If

            If Right(strCondition, 3) = "OR " Then
                strCondition = Mid(strCondition, 1, Len(strCondition) - 3)
                strCondition = strCondition & ")"
            End If

            lstResults = New ObservableCollection(Of GG_results)(Main.obj_DAO.GetResults(strCondition, "RES_date DESC"))

            If currentType = "QC" Then

                CalculateQCDeviation(lstResults)

            End If

            griResults.ItemsSource = lstResults

            'Amaguem els tabs segons convingui
            ShowTabs()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub CalculateQCDeviation(ByRef lstQCResults As ObservableCollection(Of GG_results))

        Try

            For Each res In lstQCResults

                If res.RES_result >= res.RES_low And res.RES_result <= res.RES_high Then

                    Select Case res.RES_level
                        Case 1
                            res.RES_deviation = 16.65
                        Case 2
                            res.RES_deviation = 49.95
                        Case 3
                            res.RES_deviation = 83.25
                    End Select

                ElseIf res.RES_result < res.RES_low Then

                    Select Case res.RES_level
                        Case 1
                            res.RES_deviation = 16.65 - ((res.RES_low - res.RES_result))
                            If res.RES_deviation < 0 Then
                                res.RES_deviation = 1
                            End If
                        Case 2
                            res.RES_deviation = 49.95 - ((res.RES_low - res.RES_result))
                            If res.RES_deviation < 33.3 Then
                                res.RES_deviation = 34
                            End If
                        Case 3
                            res.RES_deviation = 83.25 - ((res.RES_low - res.RES_result))
                            If res.RES_deviation < 66.6 Then
                                res.RES_deviation = 68
                            End If
                    End Select

                ElseIf res.RES_result > res.RES_high Then

                    Select Case res.RES_level
                        Case 1
                            res.RES_deviation = 16.65 + ((res.RES_result - res.RES_high))
                            If res.RES_deviation > 33.3 Then
                                res.RES_deviation = 32
                            End If
                        Case 2
                            res.RES_deviation = 49.95 + ((res.RES_result - res.RES_high))
                            If res.RES_deviation > 66.6 Then
                                res.RES_deviation = 65
                            End If
                        Case 3
                            res.RES_deviation = 83.25 + ((res.RES_result - res.RES_high))
                            If res.RES_deviation > 100 Then
                                res.RES_deviation = 99
                            End If
                    End Select

                End If

            Next

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub GetResultColor(sender As System.Object, e As System.Windows.RoutedEventArgs)

        Dim objResult As New GG_results

        Try

            objResult = DirectCast(sender.Parent.Parent.Datacontext, GG_results)

            'Segons el rang de normalitat acceptat en el mesurador marquem el resultat com a BAIX, NORMAL o ALT

            If objResult.RES_result = 0 And objResult.RES_incidence <> "" Then
                sender.Foreground = Brushes.Black
            ElseIf objResult.RES_result < objResult.RES_low Then
                sender.Foreground = New SolidColorBrush(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "LowColor")))
            ElseIf objResult.RES_result >= objResult.RES_low And objResult.RES_result <= objResult.RES_high Then
                sender.Foreground = New SolidColorBrush(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "NormalColor")))
            ElseIf objResult.RES_result > objResult.RES_high Then
                sender.Foreground = New SolidColorBrush(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub GetStatistics()

        Try

            'Obtenim els resultats que hi ha filtrats en el grid
            Dim lstItemsGrid As New List(Of GG_results)
            lstItemsGrid = griResults.Items.Cast(Of GG_results).ToList()

            'Calculem el total del grid
            Dim totalGrid As Long
            totalGrid = lstItemsGrid.Count

            'Calculem la mitja dels valors del grid
            avgGrid = 0
            If lstItemsGrid.Count <> 0 Then
                avgGrid = (From res In lstItemsGrid Select res).Sum(Function(res) res.RES_result) / lstItemsGrid.Count
            Else
                avgGrid = 0
            End If

            'Calculem la SD dels valors del grid
            SDGrid = 0
            If lstItemsGrid.Count <> 0 Then

                For Each res In lstItemsGrid
                    SDGrid = SDGrid + ((res.RES_result - avgGrid) ^ 2)
                Next

                SDGrid = Sqrt((SDGrid / totalGrid))

            Else

                SDGrid = 0

            End If

            'Calculem el CV dels valors del grid
            CVGrid = 0
            If SDGrid <> 0 And avgGrid <> 0 Then
                CVGrid = (SDGrid / avgGrid) * 100
            Else
                CVGrid = 0
            End If

            'Calculem el total de resultats
            Dim total As Long
            total = (From res In lstResults Where res.RES_type = currentType And res.TES_id_test = currentTest Select res).Count

            btnTotal.Content = My.Resources.Resource.msg_Total & ": " & totalGrid & " / " & total

            'Calculem la mitja de valors
            Dim avg As Double
            avg = (From res In lstResults Where res.RES_type = currentType And res.TES_id_test = currentTest Select res).Sum(Function(res) res.RES_result) / total

            btnAverage.Content = My.Resources.Resource.msg_Average & ": " & Format(avgGrid, "###,##0.#") & " / " & Format(avg, "###,##0.#")

            'Calculem la SD
            Dim SD As Double

            If total <> 0 Then

                For Each result In (From res In lstResults Where res.RES_type = currentType And res.TES_id_test = currentTest Select res)
                    SD = SD + ((result.RES_result - avg) ^ 2)
                Next

                SD = Sqrt((SD / total))

            Else

                SD = 0

            End If

            btnSD.Content = "SD: " & Format(SDGrid, "###,##0.#") & " / " & Format(SD, "###,##0.#")

            'Calculem el CV
            Dim CV As Double
            If SD <> 0 And avg <> 0 Then
                CV = (SD / avg) * 100
            Else
                CV = 0
            End If

            btnCV.Content = "CV: " & Format(CVGrid, "###,##0.#") & " / " & Format(CV, "###,##0.#") & "%"

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub InitializeCombos()

        'Inicialitzem els valors dels combos que ens serviran com a filtres

        Try

            ResetCombos()

            'Inicialitzem el combo dels Serveis

            Dim lstServices = From r In lstResults _
                            Where r.SER_description <> "" _
                            Order By r.SER_description _
                            Group By SER_description = r.SER_description _
                            Into Grupo = Group _
                            Order By SER_description Ascending

            cmbServices.Items.Add("")

            For Each res In lstServices

                cmbServices.Items.Add(res.SER_description)

            Next

            'Inicialitzem el combo dels Operadors

            Dim lstOperators = From r In lstResults _
                                  Where r.RES_operator <> "" _
                                  Order By r.RES_operator _
                                  Group By RES_operator = r.RES_operator _
                                  Into Grupo = Group _
                                  Order By RES_operator Ascending

            cmbOperators.Items.Add("")

            For Each res In lstOperators

                cmbOperators.Items.Add(res.RES_operator)

            Next

            'Inicialitzem el combo dels Analitzadors

            Dim lstAnalyzers = From r In lstResults _
                                 Where r.ANA_factory_number <> "" _
                                 Order By r.ANA_factory_number _
                                 Group By ANA_factory_number = r.ANA_factory_number _
                                 Into Grupo = Group _
                                 Order By ANA_factory_number Ascending

            cmbAnalyzers.Items.Add("")

            For Each res In lstAnalyzers

                cmbAnalyzers.Items.Add(res.ANA_factory_number)

            Next

            'Inicialitzem el combo dels controls

            Dim lstControls = From r In lstResults _
                                 Where r.RES_control <> "" _
                                 Order By r.RES_control _
                                 Group By RES_control = r.RES_control _
                                 Into Grupo = Group _
                                 Order By RES_control Ascending

            For Each res In lstControls

                cmbControl.Items.Add(res.RES_control)

            Next


            'Inicialitzem el combo dels nivells

            Dim lstLevels = From r In lstResults _
                                        Where r.RES_level <> "" _
                                        Order By r.RES_level _
                                        Group By RES_level = r.RES_level _
                                        Into Grupo = Group _
                                        Order By RES_level Ascending

            For Each res In lstLevels

                cmbLevel.Items.Add(res.RES_level)

            Next

            'Inicialitzem el combo de les tires

            Dim lstReactives = From r In lstResults _
                             Where r.RES_reactive <> "" _
                             Order By r.RES_reactive _
                             Group By RES_reactive = r.RES_reactive _
                             Into Grupo = Group _
                             Order By RES_reactive Ascending

            For Each res In lstReactives

                cmbReactive.Items.Add(res.RES_reactive)

            Next

            'Inicialitzem el combo dels Pacients

            Dim lstPatients = From r In lstResults _
                                 Where r.PAT_history_number <> "" _
                                 Order By r.PAT_history_number _
                                 Group By PAT_history_number = r.PAT_history_number _
                                 Into Grupo = Group _
                                 Order By PAT_history_number Ascending

            cmbPatients.Items.Add("")

            For Each res In lstPatients

                cmbPatients.Items.Add(res.PAT_history_number)

            Next

            'Inicialitzem els períodes de l'usuari

            cmbPeriods.ItemsSource = Main.lstPeriods

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub ResetCombos()

        Try

            'Eliminem tots els items dels combobox

            cmbAnalyzers.Items.Clear()
            cmbOperators.Items.Clear()
            cmbPatients.Items.Clear()
            cmbServices.Items.Clear()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Public Sub ApplyFilter()

        'Apliquem filtres sobre el grid en funció dels valors dels combos

        Try

            griResults.FilterDescriptors.Clear()


            'Apliquem el filtre de la prova
            Select Case currentType

                Case "RES"

                    If cmbResultsTests.SelectedItem IsNot Nothing Then

                        griResults.FilterDescriptors.Add(New FilterDescriptor("TES_id_test", FilterOperator.IsEqualTo, cmbResultsTests.SelectedItem.TES_id_test))

                    End If

                Case "QC"

                    If cmbQCTests.SelectedItem IsNot Nothing Then

                        griResults.FilterDescriptors.Add(New FilterDescriptor("TES_id_test", FilterOperator.IsEqualTo, cmbQCTests.SelectedItem.TES_id_test))

                    End If

            End Select

            'Apliquem el filtre sobre les hores

            Dim datStartHour As Date = Nothing
            Dim datEndHour As Date = Nothing

            If timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString = "" Then

                datStartHour = New Date(Now.Year, Now.Month, Now.Day, timFrom.SelectedTime.Value.Hours, timFrom.SelectedTime.Value.Minutes, timFrom.SelectedTime.Value.Seconds)

                griResults.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsGreaterThanOrEqualTo, datStartHour))

            End If

            If timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString <> "" Then

                datEndHour = New Date(Now.Year, Now.Month, Now.Day, timTo.SelectedTime.Value.Hours, timTo.SelectedTime.Value.Minutes, timTo.SelectedTime.Value.Seconds)

                griResults.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsLessThanOrEqualTo, datEndHour))

            End If

            If timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString <> "" Then

                datStartHour = New Date(Now.Year, Now.Month, Now.Day, timFrom.SelectedTime.Value.Hours, timFrom.SelectedTime.Value.Minutes, timFrom.SelectedTime.Value.Seconds)
                datEndHour = New Date(Now.Year, Now.Month, Now.Day, timTo.SelectedTime.Value.Hours, timTo.SelectedTime.Value.Minutes, timTo.SelectedTime.Value.Seconds)

                If datStartHour > datEndHour Then

                    Dim compositeDescriptor As New CompositeFilterDescriptor()
                    compositeDescriptor.LogicalOperator = FilterCompositionLogicalOperator.[Or]

                    Dim datStartHourcioDescriptor As New CompositeFilterDescriptor()

                    datStartHourcioDescriptor.LogicalOperator = FilterCompositionLogicalOperator.[And]
                    datStartHourcioDescriptor.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsGreaterThanOrEqualTo, datStartHour))
                    datStartHourcioDescriptor.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsLessThanOrEqualTo, New Date(Now.Year, Now.Month, Now.Day, 23, 59, 59)))

                    Dim datEndHourDescriptor As New CompositeFilterDescriptor()

                    datEndHourDescriptor.LogicalOperator = FilterCompositionLogicalOperator.[And]
                    datEndHourDescriptor.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsGreaterThanOrEqualTo, New Date(Now.Year, Now.Month, Now.Day, 0, 0, 0)))
                    datEndHourDescriptor.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsLessThanOrEqualTo, datEndHour))

                    compositeDescriptor.FilterDescriptors.Add(datStartHourcioDescriptor)
                    compositeDescriptor.FilterDescriptors.Add(datEndHourDescriptor)

                    griResults.FilterDescriptors.Add(compositeDescriptor)

                Else

                    griResults.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsGreaterThanOrEqualTo, datStartHour))
                    griResults.FilterDescriptors.Add(New FilterDescriptor("RES_hour", FilterOperator.IsLessThanOrEqualTo, datEndHour))

                End If

            End If

            'Apliquem el filtre sobre la Secció

            If SEC_id_section <> 0 Then
                griResults.FilterDescriptors.Add(New FilterDescriptor("SEC_id_section", FilterOperator.IsEqualTo, SEC_id_section))
            End If

            'Apliquem el filtre sobre el Servei

            If cmbServices.SelectedItem <> Nothing Then

                If cmbServices.SelectedItem.ToString <> "" Then

                    griResults.FilterDescriptors.Add(New FilterDescriptor("SER_description", FilterOperator.IsEqualTo, cmbServices.SelectedItem))

                End If

            End If

            'Apliquem el filtre sobre l'Operador

            If cmbOperators.SelectedItem <> Nothing Then

                If cmbOperators.SelectedItem.ToString <> "" Then

                    griResults.FilterDescriptors.Add(New FilterDescriptor("RES_operator", FilterOperator.IsEqualTo, cmbOperators.SelectedItem))

                End If

            End If

            'Apliquem el filtre sobre l'Analitzador

            If cmbAnalyzers.SelectedItem <> Nothing Then

                If cmbAnalyzers.SelectedItem.ToString <> "" Then

                    griResults.FilterDescriptors.Add(New FilterDescriptor("ANA_factory_number", FilterOperator.IsEqualTo, cmbAnalyzers.SelectedItem))

                End If

            End If

            'Apliquem el filtre sobre el Pacient

            If cmbPatients.SelectedItem <> Nothing Then

                If cmbPatients.SelectedItem.ToString <> "" Then

                    griResults.FilterDescriptors.Add(New FilterDescriptor("PAT_history_number", FilterOperator.IsEqualTo, cmbPatients.SelectedItem))

                End If

            End If

            'Apliquem el filtre sobre el Control

            If cmbControl.SelectedItem <> Nothing Then

                If cmbControl.SelectedItem.ToString <> "" Then

                    griResults.FilterDescriptors.Add(New FilterDescriptor("RES_control", FilterOperator.IsEqualTo, cmbControl.SelectedItem))
                    TabLJ.IsEnabled = True

                End If

            Else

                If cmbLevel.SelectedItem = Nothing Then
                    TabLJ.IsEnabled = False
                End If

            End If

            'Apliquem el filtre sobre el Nivell

            If cmbLevel.SelectedItem <> Nothing Then

                If cmbLevel.SelectedItem.ToString <> "" Then

                    griResults.FilterDescriptors.Add(New FilterDescriptor("RES_level", FilterOperator.IsEqualTo, cmbLevel.SelectedItem))
                    TabLJ.IsEnabled = True

                End If

            Else

                If cmbControl.SelectedItem = Nothing Then
                    TabLJ.IsEnabled = False
                End If

            End If

            'Apliquem el filtre sobre la Tira

            If cmbReactive.SelectedItem <> Nothing Then

                If cmbReactive.SelectedItem.ToString <> "" Then

                    griResults.FilterDescriptors.Add(New FilterDescriptor("RES_reactive", FilterOperator.IsEqualTo, cmbReactive.SelectedItem))

                End If

            End If

            'Calculem les estadístiques sobre els resultats del grid
            GetStatistics()

            'Redibuixem les gràfiques un cop aplicats els filtres
            RefreshChart()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub RefreshChart()

        Try

            'En funció de la gràfica actual la redibuixem
            Select Case currentChart
                Case "ChartGlobal"
                    ChartGlobal()
                Case "ChartService"
                    ChartService()
                Case "Chart24h"
                    Chart24h()
                Case "ChartLineal"
                    ChartLineal()
                Case "ChartError"
                    ChartError()
                Case "ChartQCRanges"
                    ChartQCRanges()
                Case "ChartLeveyJennings"
                    ChartLeveyJennings()
            End Select

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbAnalyzers_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbAnalyzers.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre l'Analitzador

            If cmbAnalyzers.SelectedItem Is Nothing Then
                Exit Sub
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Analyzer filtered, analyzer: " & cmbAnalyzers.Text)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbOperators_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbOperators.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre l'Operador

            If cmbOperators.SelectedItem Is Nothing Then
                Exit Sub
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Operator filtered, operator: " & cmbOperators.Text)


            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbPatients_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbPatients.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre el Pacient

            If cmbPatients.SelectedItem Is Nothing Then
                Exit Sub
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Patient filtered, patient: " & cmbPatients.Text)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbSections_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbSections.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre la Secció

            If cmbSections.SelectedItem Is Nothing Then
                SEC_id_section = 0
                Exit Sub
            Else
                SEC_id_section = cmbSections.SelectedItem.SEC_id_section
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Section filtered, section: " & cmbSections.Text)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbServices_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbServices.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre el Servei

            If cmbServices.SelectedItem Is Nothing Then
                Exit Sub
            End If

            If cmbServices.Text <> "" Then

                If currentType <> "QC" Then
                    currentChart = "ChartService"
                    tabService.IsEnabled = True
                    tabService.IsSelected = True
                End If

            Else

                tabService.IsEnabled = False

                Select Case currentType

                    Case "RES"
                        currentChart = "ChartGlobal"
                        tabGlobal.IsSelected = True
                    Case "QC"

                    Case "ERROR"
                        currentChart = "ChartError"
                        tabGlobal.IsSelected = True
                End Select

            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Service filtered, service: " & cmbServices.Text)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbLevel_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbLevel.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre el nivell de QC

            If cmbLevel.SelectedItem Is Nothing Then
                Exit Sub
            End If

            If cmbLevel.Text <> "" Then

                currentChart = "ChartLeveyJennings"
                TabLJ.IsEnabled = True
                TabLJ.IsSelected = True

            Else

                currentChart = "ChartQCRanges"
                TabLJ.IsEnabled = False
                tabQCRanges.IsSelected = True

            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Level filtered, level: " & cmbLevel.Text)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbControl_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbControl.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre el control de QC

            If cmbControl.SelectedItem Is Nothing Then
                Exit Sub
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Control filtered, control: " & cmbControl.Text)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbReactive_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbReactive.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre el reactiu de QC

            If cmbReactive.SelectedItem Is Nothing Then
                Exit Sub
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Reactive filtered, reactive: " & cmbReactive.Text)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub datFrom_DropDownClosed(sender As Object, e As System.Windows.RoutedEventArgs) Handles datFrom.DropDownClosed

        Try

            'Al tancar el combo recuperem els resultats del Servei WCF amb les noves dates

            If datFrom.SelectedDate Is Nothing Then
                Exit Sub
            End If

            InitializeResults(SEC_id_section)

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Date from filtered, date: " & Format(datFrom.SelectedDate, " dd/MM/yyyy"))

            ApplyFilter()

            'Recarreguem les possibilitats dels combos
            InitializeCombos()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub datTo_DropDownClosed(sender As Object, e As System.Windows.RoutedEventArgs) Handles datTo.DropDownClosed

        Try

            'Al tancar el combo recuperem els resultats del Servei WCF amb les noves dates

            If datTo.SelectedDate Is Nothing Then
                Exit Sub
            End If

            InitializeResults(SEC_id_section)

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Date to filtered, date: " & Format(datTo.SelectedDate, " dd/MM/yyyy"))

            ApplyFilter()

            'Recarreguem les possibilitats dels combos
            InitializeCombos()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub timFrom_DropDownClosed(sender As Object, e As System.Windows.RoutedEventArgs) Handles timFrom.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid amb el nou interval de temps

            If timFrom.SelectedTime Is Nothing Then
                Exit Sub
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Time from filtered, time: " & Format(timFrom.SelectedTime, "HH:mm"))

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub timTo_DropDownClosed(sender As Object, e As System.Windows.RoutedEventArgs) Handles timTo.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid amb el nou interval de temps

            If timTo.SelectedTime Is Nothing Then
                Exit Sub
            End If

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Time to filtered, time: " & Format(timTo.SelectedTime, "HH:mm"))

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbResultsTests_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbResultsTests.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre la Prova

            If cmbResultsTests.SelectedItem Is Nothing Then
                Exit Sub
            End If

            tabResults.IsSelected = True
            currentTest = sender.SelectedItem.TES_id_test

            InitializeRanges(Main.GetVariable("Test", sender.SelectedItem.TES_name & "_ranges"))

            tabGlobal.IsSelected = True
            currentChart = "ChartGlobal"

            VerifycurrentType("RES")

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Getting test results, test: " & sender.SelectedItem.TES_name)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbQCTests_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbQCTests.DropDownClosed

        Try

            'Al tancar el combo apliquem un filtre al grid sobre la Prova

            If cmbQCTests.SelectedItem Is Nothing Then
                Exit Sub
            End If

            tabQC.IsSelected = True
            currentTest = sender.SelectedItem.TES_id_test

            InitializeRanges(Main.GetVariable("Test", sender.SelectedItem.TES_name & "_ranges"))

            currentChart = "ChartQCRanges"
            tabQCRanges.IsSelected = True

            VerifycurrentType("QC")

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Getting test qc, test: " & sender.SelectedItem.TES_name)

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub VerifycurrentType(ByVal strType As String)

        Try

            'Comprovem si s'ha canviat el Tipus de Resultat actual per recarregar els Resultats de del Servei WCF

            If currentType <> strType Then

                currentType = strType

                InitializeResults(SEC_id_section)
                InitializeCombos()

            End If

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbPeriods_DropDownClosed(sender As Object, e As System.EventArgs) Handles cmbPeriods.DropDownClosed

        Try

            'Al seleccionar un Període horari filtem per l'hora d'inici i final del Període

            If cmbPeriods.SelectedItem Is Nothing Then
                Exit Sub
            End If

            Dim strStart As String = ""
            Dim strEnd As String = ""

            strStart = DirectCast(cmbPeriods.SelectedItem, GG_periods).PER_start_hour
            strEnd = DirectCast(cmbPeriods.SelectedItem, GG_periods).PER_end_hour

            timFrom.SelectedTime = New TimeSpan(CDate(strStart).Ticks)
            timTo.SelectedTime = New TimeSpan(CDate(strEnd).Ticks)

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Filtering period: " & cmbPeriods.SelectedItem.PER_name)

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Sub ChartGlobal()

        'Si la gràfica no conté resultats mostrem un missatge explicatiu
        If griResults.Items.Count = 0 Then

            Dim title As New ChartTitle
            title.Content = My.Resources.Resource.msg_Global

            chaGlobal.DefaultView.ChartTitle = title
            chaGlobal.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
            chaGlobal.ItemsSource = Nothing

            Exit Sub

        End If

        Dim lstCurrentResults As New ObservableCollection(Of GG_results)
        lstCurrentResults = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))

        Try

            Dim lstGroupedResults = From r In lstCurrentResults _
                                    Order By r.SER_description _
                                    Group By SER_description = r.SER_description _
                                    Into Grupo = Group _
                                    Order By SER_description

            '************Inicialitzem la configuració de la gràfica***********************************

            'Indiquem si l'eix de les X és del tipus DATE
            Dim obj_chart = New ChartCreator.GG_charts
            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver

            obj_chart.axisXIsDate = False

            'Indiquem els valors mínim i màxim de l'eix de les X (Si axisXIsDate és True es deixa en "")
            obj_chart.AxisXMaxValue = "0"
            obj_chart.AxisXMinValue = "0"

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = "0"
            obj_chart.AxisYMinValue = "0"

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = Nothing
            obj_chart.lastDate = Nothing

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = chaGlobal

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "#%{p} (#Y)"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 0

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = True

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.bottom

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.categoryAxisX = "STA_name"
            obj_chart.propertyAxisX = ""
            obj_chart.propertyAxisY = "STA_total"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.bars

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = My.Resources.Resource.msg_Global & " (" & Main.obj_DAO.GetSQL("SELECT TES_name FROM GG_tests WHERE TES_id_test=" & currentTest) & ")"
            obj_chart.titleChart = My.Resources.Resource.msg_Global

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = True
            obj_chart.YAutoRange = True

            '*******************Configuració de les series******************************

            For Each ubi In lstGroupedResults

                'Indiquem la 1ª serie (RESULTATS)

                Dim resultsCount As Long = 0
                Dim lstStatisticsPresenter As New List(Of StatisticsPresenter)

                For Each rang In Main.lstRanges

                    obj_chart.namesAxisX.Add(rang.RAN_min & " - " & rang.RAN_max)

                    resultsCount = ResultsTotalCount(currentTest, ubi.SER_description, rang.RAN_min, rang.RAN_max)

                    Dim staPres As New StatisticsPresenter
                    staPres.STA_name = rang.RAN_min & " - " & rang.RAN_max
                    staPres.STA_total = resultsCount

                    lstStatisticsPresenter.Add(staPres)

                Next

                obj_chart.lstResults.Add(lstStatisticsPresenter.Cast(Of Object))

                obj_chart.seriesColors.Add(Nothing)
                obj_chart.showTooltipSeries.Add(True)
                obj_chart.tooltipSeries.Add(My.Resources.Resource.msg_Service & ": " & ubi.SER_description)

                obj_chart.lstPointRanges.Add(New StatisticsPresenter)
                obj_chart.lstPointRanges.Add(New StatisticsPresenter)

                obj_chart.namesSeries.Add(ubi.SER_description)
                obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.stackedBars100)
                obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.circle)


            Next

            chaGlobal = obj_chart.DrawChart()
            chaGlobal.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try


    End Sub

    Sub ChartService()


        'Si la gràfica no conté resultats mostrem un missatge explicatiu
        If griResults.Items.Count = 0 Then

            Dim title As New ChartTitle
            title.Content = My.Resources.Resource.msg_Service

            chaService.DefaultView.ChartTitle = title
            chaService.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
            chaService.ItemsSource = Nothing

            Exit Sub

        End If

        Dim lstCurrentResults As New ObservableCollection(Of GG_results)
        lstCurrentResults = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))

        Try

            '************Inicialitzem la configuració de la gràfica***********************************

            'Indiquem si l'eix de les X és del tipus DATE
            Dim obj_chart As New ChartCreator.GG_charts
            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver

            obj_chart.axisXIsDate = False

            'Indiquem els valors mínim i màxim de l'eix de les X (Si axisXIsDate és True es deixa en "0")
            obj_chart.AxisXMaxValue = "0"
            obj_chart.AxisXMinValue = "0"

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = "0"
            obj_chart.AxisYMinValue = "0"

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = Nothing
            obj_chart.lastDate = Nothing

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = chaService

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "#XCAT (#Y)"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 0

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = False

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.bottom

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.categoryAxisX = "STA_name"
            obj_chart.propertyAxisX = ""
            obj_chart.propertyAxisY = "STA_total"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.pie

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = ""
            obj_chart.titleChart = My.Resources.Resource.msg_Service & " " & cmbServices.Text

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = True
            obj_chart.YAutoRange = True

            'Indiquem la 1ª serie (RESULTATS)

            Dim resultCount As Long = 0

            obj_chart.namesSeries.Add(cmbServices.Text)

            Dim lstStatisticsPresenter As New List(Of StatisticsPresenter)

            If currentType = "ERROR" Then

                Dim lstErrors = From r In lstCurrentResults _
                           Where (r.TES_id_test = currentTest And r.SER_description = cmbServices.Text) _
                           Order By r.RES_incidence _
                           Group By RES_incidence = r.RES_incidence _
                           Into Grupo = Group _
                           Order By RES_incidence

                For Each er In lstErrors

                    Dim stringError As String
                    stringError = er.RES_incidence

                    resultCount = (From r In lstCurrentResults Where r.RES_incidence = stringError Select r).Count

                    Dim staPres As New StatisticsPresenter
                    staPres.STA_name = stringError
                    staPres.STA_total = resultCount

                    obj_chart.namesAxisX.Add(stringError)
                    obj_chart.seriesColors.Add(Nothing)

                    lstStatisticsPresenter.Add(staPres)

                Next

                obj_chart.tooltipSeries.Add(My.Resources.Resource.msg_Errors)

            Else

                For Each rang In Main.lstRanges

                    resultCount = ResultsTotalCount(currentTest, cmbServices.Text, rang.RAN_min, rang.RAN_max)

                    Dim staPres As New StatisticsPresenter
                    staPres.STA_name = rang.RAN_min & " - " & rang.RAN_max
                    staPres.STA_total = resultCount

                    If resultCount <> 0 Then

                        obj_chart.namesAxisX.Add(rang.RAN_min & " - " & rang.RAN_max)
                        obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(rang.RAN_color))

                        lstStatisticsPresenter.Add(staPres)

                    End If

                Next

                obj_chart.tooltipSeries.Add("#XCAT")

            End If

            obj_chart.showTooltipSeries.Add(True)

            obj_chart.lstResults.Add(lstStatisticsPresenter.Cast(Of Object)())

            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.pie)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.circle)

            obj_chart.lstPointRanges.Add(New StatisticsPresenter)
            obj_chart.lstPointRanges.Add(New StatisticsPresenter)

            chaService = obj_chart.DrawChart()
            chaService.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Sub Chart24h()

        'Si la gràfica no conté resultats mostrem un missatge explicatiu
        If griResults.Items.Count = 0 Then

            Dim title As New ChartTitle
            title.Content = "24h"

            cha24h.DefaultView.ChartTitle = title
            cha24h.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
            cha24h.ItemsSource = Nothing

            Exit Sub

        End If

        Dim lstCurrentResults As New ObservableCollection(Of GG_results)
        lstCurrentResults = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))

        Try

            Dim lastDate = (From r In lstCurrentResults _
                      Select r.RES_hour).Max

            Dim firstDate = (From r In lstCurrentResults _
                           Select r.RES_hour).Min

            '************Inicialitzem la configuració de la gràfica***********************************

            'Indiquem si l'eix de les X és del tipus DATE
            Dim obj_chart As New ChartCreator.GG_charts
            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver

            obj_chart.axisXIsDate = True

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = False
            obj_chart.YAutoRange = False

            'Indiquem els valors mínim i màxim de l'eix de les X (Si axisXIsDate és True es deixa en "0")
            obj_chart.AxisXMaxValue = "0"
            obj_chart.AxisXMinValue = "0"

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = GetTestRange("HIGH")
            obj_chart.AxisYMinValue = GetTestRange("LOW")

            'Indiquem els Step dels eixos (Si axisXIsDate és True no es tenen en compte)
            obj_chart.AxisXStep = 0
            obj_chart.AxisYStep = GetTestStep()

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = firstDate
            obj_chart.lastDate = lastDate

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = cha24h

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "#VAL{HH:mm}"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 50

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = False

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.right

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.propertyAxisX = "RES_hour"
            obj_chart.propertyAxisY = "RES_result"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.points24h

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = "24h"
            obj_chart.titleChart = "24h"

            '*******************Configuració de les series******************************

            Dim values As New List(Of Double)

            Dim counter As Integer = 0

            For Each peri In Main.lstPeriods

                values = New List(Of Double)

                'Indiquem la 1ª serie (PERIODES)
                obj_chart.namesSeries.Add(counter)
                obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.verticalRange)
                obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
                obj_chart.seriesColors.Add(System.Windows.Media.Colors.LightGray)
                values.Add(CInt(Mid(peri.PER_start_hour, 1, 2)))
                obj_chart.lstValues.Add(values)
                obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
                obj_chart.lstPointRanges.Add(New GG_results)
                obj_chart.lstPointRanges.Add(New GG_results)
                obj_chart.showTooltipSeries.Add(False)
                obj_chart.tooltipSeries.Add("")

                counter = counter + 1

            Next

            values = New List(Of Double)

            'Indiquem la 2ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(Main.GetVariable("Test", Main.obj_DAO.GetSQL("SELECT TES_name FROM GG_tests WHERE TES_id_test=" & currentTest) & "_max")))

            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 3ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "LowColor")))
            values.Add(CDbl(Main.GetVariable("Test", Main.obj_DAO.GetSQL("SELECT TES_name FROM GG_tests WHERE TES_id_test=" & currentTest) & "_min")))

            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 4ª serie (RESULTATS)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.results)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.circle)
            obj_chart.seriesColors.Add(System.Windows.Media.Colors.Black)
            values.Add(CDbl(0))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(lstCurrentResults.Cast(Of Object)())
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(True)

            Dim resultsTooltip As String = GetItemTooltip()

            obj_chart.tooltipSeries.Add(resultsTooltip)

            cha24h = obj_chart.DrawChart()
            cha24h.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Sub ChartLineal()

        'Si la gràfica no conté resultats mostrem un missatge explicatiu
        If griResults.Items.Count = 0 Then

            Dim title As New ChartTitle
            title.Content = My.Resources.Resource.msg_Results & " (" & Main.obj_DAO.GetSQL("SELECT TES_name FROM GG_tests WHERE TES_id_test=" & currentTest) & ")"

            chaLineal.DefaultView.ChartTitle = title
            chaLineal.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
            chaLineal.ItemsSource = Nothing

            Exit Sub

        End If

        Dim lstCurrentResults As New ObservableCollection(Of GG_results)
        lstCurrentResults = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))

        Try

            Dim lastDate = (From r In lstCurrentResults _
                           Select r.RES_date).Max

            Dim firstDate = (From r In lstCurrentResults _
                           Select r.RES_date).Min


            '************Inicialitzem la configuració de la gràfica***********************************

            Dim obj_chart As New ChartCreator.GG_charts
            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver

            'Indiquem si l'eix de les X és del tipus DATE
            obj_chart.axisXIsDate = True

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = False
            obj_chart.YAutoRange = False

            'Indicamos valor max y min del eje de las X (Si axisXIsDate es True dejar en "")
            obj_chart.AxisXMaxValue = ""
            obj_chart.AxisXMinValue = ""

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = GetTestRange("HIGH")
            obj_chart.AxisYMinValue = GetTestRange("LOW")

            'Indiquem els Step dels eixos (Si axisXIsDate és True no es tenen en compte)

            obj_chart.AxisXStep = ChartCreator.GG_charts.CalculateStep(firstDate, lastDate)
            obj_chart.AxisYStep = GetTestStep()

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = firstDate
            obj_chart.lastDate = lastDate

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = chaLineal

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "dd/MM/yyyy"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 75

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = False

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.right

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.propertyAxisX = "RES_date"
            obj_chart.propertyAxisY = "RES_result"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.points

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = ""
            obj_chart.titleChart = My.Resources.Resource.msg_Results & " (" & Main.obj_DAO.GetSQL("SELECT TES_name FROM GG_tests WHERE TES_id_test=" & currentTest) & ")"

            '*******************Configuració de les series******************************

            Dim values As New List(Of Double)

            'Indiquem la 1ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(Main.GetVariable("Test", Main.obj_DAO.GetSQL("SELECT TES_name FROM GG_tests WHERE TES_id_test=" & currentTest) & "_max")))

            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 2ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "LowColor")))
            values.Add(CDbl(Main.GetVariable("Test", Main.obj_DAO.GetSQL("SELECT TES_name FROM GG_tests WHERE TES_id_test=" & currentTest) & "_min")))

            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 3ª serie (RESULTATS)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.results)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.circle)
            obj_chart.seriesColors.Add(System.Windows.Media.Colors.Black)
            values.Add(CDbl(0))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(lstCurrentResults.Cast(Of Object)())
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(True)

            Dim resultsTooltip As String = GetItemTooltip()

            obj_chart.tooltipSeries.Add(resultsTooltip)

            chaLineal = obj_chart.DrawChart()
            chaLineal.Rebind()


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Sub ChartError()

        'Si la gràfica no conté resultats mostrem un missatge explicatiu
        If griResults.Items.Count = 0 Then

            Dim title As New ChartTitle
            title.Content = My.Resources.Resource.msg_Errors

            chaGlobal.DefaultView.ChartTitle = title
            chaGlobal.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
            chaGlobal.ItemsSource = Nothing

            Exit Sub

        End If

        Dim lstCurrentResults As New ObservableCollection(Of GG_results)
        lstCurrentResults = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))

        Try

            Dim lstGroupedResults = From r In lstCurrentResults _
                                    Order By r.SER_description _
                                    Group By SER_description = r.SER_description _
                                    Into Grupo = Group _
                                    Order By SER_description

            '************Inicialitzem la configuració de la gràfica***********************************

            'Indiquem si l'eix de les X és del tipus DATE
            Dim obj_chart As New ChartCreator.GG_charts
            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver

            obj_chart.axisXIsDate = False

            'Indicamos valor max y min del eje de las X (Si anterior es True dejar en "")
            obj_chart.AxisXMaxValue = "0"
            obj_chart.AxisXMinValue = "0"

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = "0"
            obj_chart.AxisYMinValue = "0"

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = Nothing
            obj_chart.lastDate = Nothing

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = chaGlobal

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "#%{p} (#Y)"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 0

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = False

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.bottom

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.categoryAxisX = "STA_name"
            obj_chart.propertyAxisX = ""
            obj_chart.propertyAxisY = "STA_total"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.bars

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = ""
            obj_chart.titleChart = My.Resources.Resource.msg_Errors

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = True
            obj_chart.YAutoRange = True

            '*******************Configuració de les series******************************

            Dim lstStatisticsPresenter As New List(Of StatisticsPresenter)

            'Indiquem la 1ª serie (RESULTATS)

            For Each ubi In lstGroupedResults

                Dim totalCount = (From r In lstCurrentResults _
                                  Where r.SER_description = ubi.SER_description _
                                  Select r).Count

                Dim staPres As New StatisticsPresenter
                staPres.STA_name = ubi.SER_description
                staPres.STA_total = totalCount

                lstStatisticsPresenter.Add(staPres)

            Next

            obj_chart.lstResults.Add(lstStatisticsPresenter.Cast(Of Object)())

            obj_chart.namesSeries.Add(My.Resources.Resource.msg_Errors)
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.bars)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.circle)

            obj_chart.showTooltipSeries.Add(True)
            obj_chart.tooltipSeries.Add(My.Resources.Resource.msg_Total & ": #Y{##0.##}")

            obj_chart.namesAxisX.Add(My.Resources.Resource.msg_Errors)
            obj_chart.seriesColors.Add(Nothing)

            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)

            chaGlobal = obj_chart.DrawChart()
            chaGlobal.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try


    End Sub

    Sub ChartQCRanges()

        'Si la gràfica no conté resultats mostrem un missatge explicatiu
        If griResults.Items.Count = 0 Then

            Dim title As New ChartTitle
            title.Content = My.Resources.Resource.msg_QCRanges

            chaQCRanges.DefaultView.ChartTitle = title
            chaQCRanges.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
            chaQCRanges.ItemsSource = Nothing

            Exit Sub

        End If

        Dim lstCurrentResults As New ObservableCollection(Of GG_results)
        lstCurrentResults = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))

        Try


            Dim fechaMax = (From r In lstCurrentResults _
                           Select r.RES_date).Max

            Dim fechaMin = (From r In lstCurrentResults _
                           Select r.RES_date).Min


            '************Inicialitzem la configuració de la gràfica***********************************

            'Indiquem si l'eix de les X és del tipus DATE
            Dim obj_chart = New ChartCreator.GG_charts
            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver

            obj_chart.axisXIsDate = True

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = False
            obj_chart.YAutoRange = False

            'Indicamos valor max y min del eje de las X (Si axisXIsDate es True dejar en "")
            obj_chart.AxisXMaxValue = ""
            obj_chart.AxisXMinValue = ""

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = "100"
            obj_chart.AxisYMinValue = "0"

            'Indiquem els Step dels eixos (Si axisXIsDate és True no es tenen en compte)
            obj_chart.AxisXStep = ChartCreator.GG_charts.CalculateStep(fechaMin, fechaMax)
            obj_chart.AxisYStep = GetTestStep()

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = fechaMin
            obj_chart.lastDate = fechaMax

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = chaQCRanges

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "dd/MM/yyyy"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 75

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = False

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.right

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.propertyAxisX = "RES_date"
            obj_chart.propertyAxisY = "RES_deviation"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.points

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = My.Resources.Resource.msg_QCRanges
            obj_chart.titleChart = My.Resources.Resource.msg_QCRanges

            '*******************Configuració de les series******************************

            Dim transparentColor As String
            Dim values As New List(Of Double)

            'Indiquem la 1ª serie (RANG)
            obj_chart.namesSeries.Add("Level 1")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "LowColor")))
            values.Add(CDbl(16.65))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 1ª serie (RANG)
            obj_chart.namesSeries.Add("Level 1")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.qcout)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            transparentColor = GetTransparentColor(Main.GetVariable("General", "LowColor"), "#30")
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(transparentColor))
            values.Add(CDbl(16.65))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 2ª serie (RANG)
            obj_chart.namesSeries.Add("Level 2")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "NormalColor")))
            values.Add(CDbl(49.95))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 2ª serie (RANG)
            obj_chart.namesSeries.Add("Level 2")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.qcout)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            transparentColor = GetTransparentColor(Main.GetVariable("General", "NormalColor"), "#30")
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(transparentColor))
            values.Add(CDbl(49.95))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 3ª serie (RANG)
            obj_chart.namesSeries.Add("Level 3")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(83.25))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 3ª serie (RANG)
            obj_chart.namesSeries.Add("Level 3")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.qcout)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            transparentColor = GetTransparentColor(Main.GetVariable("General", "HighColor"), "#30")
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(transparentColor))
            values.Add(CDbl(83.25))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            Dim lstLevel1 = From r In lstCurrentResults _
                          Where r.RES_level = "1" _
                          Select r

            'Indiquem la 4ª serie (RESULTATS NIVELL 1)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.results)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.diamond)
            obj_chart.seriesColors.Add(System.Windows.Media.Colors.Black)
            values.Add(0)
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(lstLevel1.Cast(Of Object)())
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(True)

            Dim resultsTooltip As String = GetItemTooltip()

            obj_chart.tooltipSeries.Add(resultsTooltip)

            values = New List(Of Double)

            Dim lstLevel2 = From r In lstCurrentResults _
                          Where r.RES_level = "2" _
                          Select r

            'Indiquem la 5ª serie (RESULTATS NIVELL 2)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.results)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.star5)
            obj_chart.seriesColors.Add(System.Windows.Media.Colors.Black)
            values.Add(0)
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(lstLevel2.Cast(Of Object)())
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(True)

            resultsTooltip = GetItemTooltip()

            obj_chart.tooltipSeries.Add(resultsTooltip)

            values = New List(Of Double)


            Dim lstLevel3 = From r In lstCurrentResults _
                          Where r.RES_level = "3" _
                          Select r

            'Indiquem la 6ª serie (RESULTATS NIVELL 3)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.results)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.triangle)
            obj_chart.seriesColors.Add(System.Windows.Media.Colors.Black)
            values.Add(0)
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(lstLevel3.Cast(Of Object)())
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(True)

            resultsTooltip = GetItemTooltip()

            obj_chart.tooltipSeries.Add(resultsTooltip)

            chaQCRanges = obj_chart.DrawChart()
            chaQCRanges.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub


    Sub ChartLeveyJennings()

        'Si la gràfica no conté resultats mostrem un missatge explicatiu
        If griResults.Items.Count = 0 Then

            Dim title As New ChartTitle
            title.Content = My.Resources.Resource.msg_QCRanges

            chaLeveyJennings.DefaultView.ChartTitle = title
            chaLeveyJennings.DefaultView.ChartArea.NoDataString = My.Resources.Resource.msg_NoData
            chaLeveyJennings.ItemsSource = Nothing

            Exit Sub

        End If

        Dim lstCurrentResults As New ObservableCollection(Of GG_results)
        lstCurrentResults = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))

        Try


            Dim fechaMax = (From r In lstCurrentResults _
                           Select r.RES_date).Max

            Dim fechaMin = (From r In lstCurrentResults _
                           Select r.RES_date).Min

            Dim values As New List(Of Double)

            '************Inicialitzem la configuració de la gràfica***********************************

            'Indiquem si l'eix de les X és del tipus DATE
            Dim obj_chart = New ChartCreator.GG_charts
            obj_chart.chartTheme = ChartCreator.GG_charts.chartThemes.Office_Silver

            obj_chart.axisXIsDate = True

            'Indiquem si es calcula automàticament els rangs dels eixos X i Y
            obj_chart.XAutoRange = False
            obj_chart.YAutoRange = False

            'Indicamos valor max y min del eje de las X (Si axisXIsDate es True dejar en "")
            obj_chart.AxisXMaxValue = ""
            obj_chart.AxisXMinValue = ""

            'Indiquem el valor màxim i mínim de l'eix de les Y
            obj_chart.AxisYMaxValue = avgGrid + SDGrid + SDGrid + SDGrid + SDGrid
            obj_chart.AxisYMinValue = avgGrid - SDGrid - SDGrid - SDGrid - SDGrid

            'Indiquem els Step dels eixos (Si axisXIsDate és True no es tenen en compte)
            obj_chart.AxisXStep = ChartCreator.GG_charts.CalculateStep(fechaMin, fechaMax)
            obj_chart.AxisYStep = SDGrid

            'Indiquem la data d'inici i de final de l'eix de les X (Si axisXIsDate és False deixar en "")
            obj_chart.firstDate = fechaMin
            obj_chart.lastDate = fechaMax

            'Indiquem la gràfica sobre la que es realitzen els canvis
            obj_chart.currentChart = chaLeveyJennings

            'Indiquem el format de les etiquetes dels punts
            obj_chart.labelFormat = "dd/MM/yyyy"

            'Indiquem el grau de rotació de l'eix X
            obj_chart.labelRotationAngle = 75

            'Indiquem si es mostra o no la llegenda
            obj_chart.showLegend = False

            'Indiquem si es mostra o no la quadrícula horitzontal i vertical de la gràfica
            obj_chart.showLinesAxisX = False
            obj_chart.showLinesAxisY = False

            'Indiquem el missatge que apareix quan no hi ha resultats
            obj_chart.noDataString = My.Resources.Resource.msg_NoData

            'Indiquem en quin lloc apareix la llegenda
            obj_chart.legendPosition = ChartCreator.GG_charts.legendPositions.right

            'Indiquem quines propietats de l'objectt formen els eixos X i Y
            obj_chart.propertyAxisX = "RES_date"
            obj_chart.propertyAxisY = "RES_result"

            'Indiquem el tipus de gràfica que volem dibuixar
            obj_chart.chartType = ChartCreator.GG_charts.chartTypes.points

            'Indiquem el nom de la llegenda
            obj_chart.titleLegend = "Levey-Jennings"
            obj_chart.titleChart = "Levey-Jennings"

            '*******************Configuració de les series******************************
            values = New List(Of Double)

            'Indiquem la 1ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "NormalColor")))
            values.Add(CDbl(avgGrid))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 2ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(avgGrid + SDGrid))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 3ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(avgGrid + SDGrid + SDGrid))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 4ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(avgGrid + SDGrid + SDGrid + SDGrid))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 5ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(avgGrid - SDGrid))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 6ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(avgGrid - SDGrid - SDGrid))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 7ª serie (RANG)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.horizontalRange)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.hexagon)
            obj_chart.seriesColors.Add(System.Windows.Media.ColorConverter.ConvertFromString(Main.GetVariable("General", "HighColor")))
            values.Add(CDbl(avgGrid - SDGrid - SDGrid - SDGrid))
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(Nothing) 'Si és un rang afegim una llista de resultats buida
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(False)
            obj_chart.tooltipSeries.Add("")

            values = New List(Of Double)

            'Indiquem la 8ª serie (RESULTATS)
            obj_chart.namesSeries.Add("")
            obj_chart.typesSeries.Add(ChartCreator.GG_charts.serie.results)
            obj_chart.pointSeries.Add(ChartCreator.GG_charts.point.circle)
            obj_chart.seriesColors.Add(System.Windows.Media.Colors.Black)
            values.Add(0)
            obj_chart.lstValues.Add(values)
            obj_chart.lstResults.Add(lstCurrentResults.Cast(Of Object)())
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.lstPointRanges.Add(New GG_results)
            obj_chart.showTooltipSeries.Add(True)

            Dim resultsTooltip As String = GetItemTooltip()

            obj_chart.tooltipSeries.Add(resultsTooltip)

            chaLeveyJennings = obj_chart.DrawChart()
            chaLeveyJennings.Rebind()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Function ResultsTotalCount(ByVal TES_id_test As Integer, ByVal SER_description As String, ByVal RAN_min As Integer, ByVal RAN_max As Integer) As Long

        Dim resultsCount As Long = 0

        Try

            'Comprovem el filtre de dates

            If datFrom.SelectedDate.ToString = "" And datTo.SelectedDate.ToString = "" Then

                'Comprovem el filtre d'hores

                If timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_hour.TimeOfDay.Ticks >= timFrom.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString <> "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_hour.TimeOfDay.Ticks <= timTo.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString <> "" Then

                    Dim horaIni = New Date(Now.Year, Now.Month, Now.Day, timFrom.SelectedTime.Value.Hours, timFrom.SelectedTime.Value.Minutes, timFrom.SelectedTime.Value.Seconds)
                    Dim horaFin = New Date(Now.Year, Now.Month, Now.Day, timTo.SelectedTime.Value.Hours, timTo.SelectedTime.Value.Minutes, timTo.SelectedTime.Value.Seconds)

                    If horaIni > horaFin Then

                        resultsCount = (From r In lstResults _
                                           Where (r.RES_result > RAN_min And r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description _
                                                  And ((r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= New Date(Now.Year, Now.Month, Now.Day, 23, 59, 59).TimeOfDay.Ticks) Or (r.RES_hour.TimeOfDay.Ticks >= New Date(Now.Year, Now.Month, Now.Day, 0, 0, 0).TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks)))).Count

                    Else

                        resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks) _
                        Select r).Count

                    End If

                End If

            ElseIf datFrom.SelectedDate.ToString <> "" And datTo.SelectedDate.ToString = "" Then

                If timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_hour.TimeOfDay.Ticks >= timFrom.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString <> "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_hour.TimeOfDay.Ticks <= timTo.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString <> "" Then

                    Dim horaIni = New Date(Now.Year, Now.Month, Now.Day, timFrom.SelectedTime.Value.Hours, timFrom.SelectedTime.Value.Minutes, timFrom.SelectedTime.Value.Seconds)
                    Dim horaFin = New Date(Now.Year, Now.Month, Now.Day, timTo.SelectedTime.Value.Hours, timTo.SelectedTime.Value.Minutes, timTo.SelectedTime.Value.Seconds)

                    If horaIni > horaFin Then

                        resultsCount = (From r In lstResults _
                                           Where (r.RES_result > RAN_min And r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate _
                                                  And ((r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= New Date(Now.Year, Now.Month, Now.Day, 23, 59, 59).TimeOfDay.Ticks) Or (r.RES_hour.TimeOfDay.Ticks >= New Date(Now.Year, Now.Month, Now.Day, 0, 0, 0).TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks)))).Count

                    Else

                        resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks) _
                        Select r).Count

                    End If

                End If


            ElseIf datFrom.SelectedDate.ToString = "" And datTo.SelectedDate.ToString <> "" Then

                If timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date <= datTo.SelectedDate.Value.AddDays(1)) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) And r.RES_hour.TimeOfDay.Ticks >= timFrom.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString <> "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) And r.RES_hour.TimeOfDay.Ticks <= timTo.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString <> "" Then

                    Dim horaIni = New Date(Now.Year, Now.Month, Now.Day, timFrom.SelectedTime.Value.Hours, timFrom.SelectedTime.Value.Minutes, timFrom.SelectedTime.Value.Seconds)
                    Dim horaFin = New Date(Now.Year, Now.Month, Now.Day, timTo.SelectedTime.Value.Hours, timTo.SelectedTime.Value.Minutes, timTo.SelectedTime.Value.Seconds)

                    If horaIni > horaFin Then

                        resultsCount = (From r In lstResults _
                                           Where (r.RES_result > RAN_min And r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) _
                                                  And ((r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= New Date(Now.Year, Now.Month, Now.Day, 23, 59, 59).TimeOfDay.Ticks) Or (r.RES_hour.TimeOfDay.Ticks >= New Date(Now.Year, Now.Month, Now.Day, 0, 0, 0).TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks)))).Count

                    Else

                        resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) And r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks) _
                        Select r).Count

                    End If

                End If

            ElseIf datFrom.SelectedDate.ToString <> "" And datTo.SelectedDate.ToString <> "" Then

                If timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_date <= datTo.SelectedDate.Value.AddDays(1)) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString = "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) And r.RES_hour.TimeOfDay.Ticks >= timFrom.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString = "" And timTo.SelectedTime.ToString <> "" Then

                    resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) And r.RES_hour.TimeOfDay.Ticks <= timTo.SelectedTime.Value.Ticks) _
                        Select r).Count

                ElseIf timFrom.SelectedTime.ToString <> "" And timTo.SelectedTime.ToString <> "" Then

                    Dim horaIni = New Date(Now.Year, Now.Month, Now.Day, timFrom.SelectedTime.Value.Hours, timFrom.SelectedTime.Value.Minutes, timFrom.SelectedTime.Value.Seconds)
                    Dim horaFin = New Date(Now.Year, Now.Month, Now.Day, timTo.SelectedTime.Value.Hours, timTo.SelectedTime.Value.Minutes, timTo.SelectedTime.Value.Seconds)

                    If horaIni > horaFin Then

                        resultsCount = (From r In lstResults _
                                           Where (r.RES_result > RAN_min And r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) _
                                                  And ((r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= New Date(Now.Year, Now.Month, Now.Day, 23, 59, 59).TimeOfDay.Ticks) Or (r.RES_hour.TimeOfDay.Ticks >= New Date(Now.Year, Now.Month, Now.Day, 0, 0, 0).TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks)))).Count

                    Else

                        resultsCount = (From r In lstResults _
                        Where (r.RES_result > RAN_min And r.RES_result <= RAN_max And r.TES_id_test = TES_id_test And r.SER_description = SER_description And r.RES_date >= datFrom.SelectedDate And r.RES_date <= datTo.SelectedDate.Value.AddDays(1) And r.RES_hour.TimeOfDay.Ticks >= horaIni.TimeOfDay.Ticks And r.RES_hour.TimeOfDay.Ticks <= horaFin.TimeOfDay.Ticks) _
                        Select r).Count

                    End If

                End If

            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return resultsCount

    End Function

    Private Function GetTestRange(ByVal type As String) As Integer

        Dim value As Integer

        Try

            'Retornem el rang de valors acceptat segons la Prova

            Select Case currentTest
                Case 1

                    If type = "LOW" Then
                        value = 0
                    Else
                        value = 600
                    End If

                Case 2

                    If type = "LOW" Then
                        value = 0
                    Else
                        value = 12
                    End If

                Case 3

                    If type = "LOW" Then
                        value = 0
                    Else
                        value = 20
                    End If

            End Select

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return value

    End Function

    Private Function GetTestStep() As Integer

        Dim value As Integer

        Try

            'Retornem la longitud de les divisions de l'eix Y segons la Prova

            Select Case currentTest
                Case 1

                    value = 100

                Case 2

                    value = 1

                Case 3

                    value = 2

            End Select

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return value

    End Function

    Private Sub ShowTabs()

        Try

            'Segons el tipus mostrem o amaguem els tabs i filtres

            Select Case currentType

                Case "RES"

                    tabQCRanges.Visibility = Windows.Visibility.Collapsed
                    TabLJ.Visibility = Windows.Visibility.Collapsed
                    tabGlobal.Visibility = Windows.Visibility.Visible
                    tab24h.Visibility = Windows.Visibility.Visible
                    tabService.Visibility = Windows.Visibility.Visible
                    tabLineal.Visibility = Windows.Visibility.Visible

                    griResults.Columns("columnPatient").IsVisible = True
                    griResults.Columns("columnError").IsVisible = False
                    griResults.Columns("columnReactive").IsVisible = False
                    griResults.Columns("columnControl").IsVisible = False
                    griResults.Columns("columnLevel").IsVisible = False

                    canQCOptions.Visibility = Windows.Visibility.Hidden

                    cmbReactive.Text = ""
                    cmbControl.Text = ""
                    cmbLevel.Text = ""

                Case "QC"

                    tabQCRanges.Visibility = Windows.Visibility.Visible
                    TabLJ.Visibility = Windows.Visibility.Visible
                    tabGlobal.Visibility = Windows.Visibility.Collapsed
                    tab24h.Visibility = Windows.Visibility.Collapsed
                    tabService.Visibility = Windows.Visibility.Collapsed
                    tabLineal.Visibility = Windows.Visibility.Collapsed

                    griResults.Columns("columnPatient").IsVisible = False
                    griResults.Columns("columnError").IsVisible = False
                    griResults.Columns("columnReactive").IsVisible = True
                    griResults.Columns("columnControl").IsVisible = True
                    griResults.Columns("columnLevel").IsVisible = True

                    canQCOptions.Visibility = Windows.Visibility.Visible

                Case "ERROR"

                    tabQCRanges.Visibility = Windows.Visibility.Collapsed
                    TabLJ.Visibility = Windows.Visibility.Collapsed
                    tabGlobal.Visibility = Windows.Visibility.Visible
                    tab24h.Visibility = Windows.Visibility.Collapsed
                    tabService.Visibility = Windows.Visibility.Visible
                    tabLineal.Visibility = Windows.Visibility.Collapsed

                    griResults.Columns("columnPatient").IsVisible = True
                    griResults.Columns("columnError").IsVisible = True
                    griResults.Columns("columnReactive").IsVisible = True
                    griResults.Columns("columnControl").IsVisible = True
                    griResults.Columns("columnLevel").IsVisible = True

                    canQCOptions.Visibility = Windows.Visibility.Hidden

                    cmbReactive.Text = ""
                    cmbControl.Text = ""
                    cmbLevel.Text = ""

            End Select

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Function GetTransparentColor(ByVal color As String, ByVal transparencia As String) As String

        Try

            'Apliquem la transparència al color que es passa per paràmetre

            color = Mid(color, 4)

            color = transparencia & color


        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return color

    End Function

    Private Function GetItemTooltip() As String

        Dim strToolTip As String = ""

        Try

            'En funció del tipus de resultat muntem un tooltip o un altre

            If currentType = "RES" Then

                strToolTip = My.Resources.Resource.msg_Date & ": #RES_date{dd/MM/yyyy}" & vbCrLf & _
                My.Resources.Resource.msg_Hour & ": #RES_hour{HH:mm:ss}" & vbCrLf & _
                My.Resources.Resource.msg_Result & ": #Y{##0.##} " & Main.obj_DAO.GetTests("TES_id_test=" & currentTest, "")(0).UNI_description & vbCrLf & _
                My.Resources.Resource.msg_Patient & ": #PAT_history_number" & vbCrLf & _
                My.Resources.Resource.msg_Service & ": #SER_description"

            ElseIf currentType = "QC" Then

                strToolTip = My.Resources.Resource.msg_Date & ": #RES_date{dd/MM/yyyy}" & vbCrLf & _
                My.Resources.Resource.msg_Hour & ": #RES_hour{HH:mm:ss}" & vbCrLf & _
                My.Resources.Resource.msg_Low & ": #RES_low{##0.##}" & vbCrLf & _
                My.Resources.Resource.msg_Result & ": #RES_result{##0.##} " & Main.obj_DAO.GetTests("TES_id_test=" & currentTest, "")(0).UNI_description & vbCrLf & _
                My.Resources.Resource.msg_High & ": #RES_high{##0.##}" & vbCrLf & _
                My.Resources.Resource.msg_Reactive & ": #RES_reactive" & vbCrLf & _
                My.Resources.Resource.msg_Control & ": #RES_control" & vbCrLf & _
                My.Resources.Resource.msg_Level & ": #RES_level" & vbCrLf & _
                My.Resources.Resource.msg_Analyzer & ": #ANA_factory_number" & vbCrLf & _
                My.Resources.Resource.msg_Service & ": #SER_description"

            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return strToolTip

    End Function

    Private Sub tab24h_GotFocus(sender As Object, e As System.Windows.RoutedEventArgs) Handles tab24h.GotFocus

        Try

            'Recarreguem la gráfica

            currentChart = "Chart24h"

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub tabLineal_GotFocus(sender As Object, e As System.Windows.RoutedEventArgs) Handles tabLineal.GotFocus

        Try

            'Recarreguem la gráfica

            currentChart = "ChartLineal"

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub tabGlobal_GotFocus(sender As Object, e As System.Windows.RoutedEventArgs) Handles tabGlobal.GotFocus

        Try

            'Recarreguem la gráfica

            If currentType = "RES" Then
                currentChart = "ChartGlobal"
            ElseIf currentType = "ERROR" Then
                currentChart = "ChartError"
            End If

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub tabService_GotFocus(sender As Object, e As System.Windows.RoutedEventArgs) Handles tabService.GotFocus

        Try

            'Recarreguem la gráfica

            currentChart = "ChartService"

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub tabErrors_GotFocus(sender As Object, e As System.Windows.RoutedEventArgs) Handles tabErrors.GotFocus

        Try

            'Recarreguem la gráfica

            currentChart = "ChartError"

            tabGlobal.IsSelected = True

            VerifycurrentType("ERROR")

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub tabQCRanges_GotFocus(sender As Object, e As System.Windows.RoutedEventArgs) Handles tabQCRanges.GotFocus

        Try

            'Recarreguem la gráfica

            currentChart = "ChartQCRanges"

            tabQCRanges.IsSelected = True

            VerifycurrentType("QC")

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbAnalyzers_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbAnalyzers.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbAnalyzers.Text = ""

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbControl_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbControl.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbControl.Text = ""

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbLevel_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbLevel.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbLevel.Text = ""

            currentChart = "ChartQCRanges"
            TabLJ.IsEnabled = False
            tabQCRanges.IsSelected = True

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbOperators_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbOperators.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbOperators.Text = ""

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbPatients_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbPatients.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbPatients.Text = ""

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbPeriods_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbPeriods.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbPeriods.Text = ""
            timFrom.SelectedTime = Nothing
            timTo.SelectedTime = Nothing

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbReactive_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbReactive.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbReactive.Text = ""

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbSections_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbSections.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbSections.Text = ""

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub cmbServices_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles cmbServices.MouseRightButtonDown

        Try

            'Eliminem el filtre del combo i recarreguem els resultats

            cmbServices.Text = ""

            tabService.IsEnabled = False

            Select Case currentType

                Case "RES"
                    currentChart = "ChartGlobal"
                    tabGlobal.IsSelected = True
                Case "QC"

                Case "ERROR"
                    currentChart = "ChartError"
                    tabGlobal.IsSelected = True
            End Select

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub datFrom_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles datFrom.MouseRightButtonDown

        Try

            'Eliminem la data i recuperem els resultats del Servei WCF amb les noves dates

            datFrom.SelectedDate = Nothing

            InitializeResults(SEC_id_section)
            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub datTo_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles datTo.MouseRightButtonDown

        Try

            'Eliminem la data i recuperem els resultats del Servei WCF amb les noves dates

            datTo.SelectedDate = Nothing

            InitializeResults(SEC_id_section)
            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub timFrom_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles timFrom.MouseRightButtonDown

        Try

            'Eliminem l'hora i recarreguem els resultats

            timFrom.SelectedTime = Nothing

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub timTo_MouseRightButtonDown(sender As Object, e As System.Windows.Input.MouseButtonEventArgs) Handles timTo.MouseRightButtonDown

        Try

            'Eliminem l'hora i recarreguem els resultats

            timTo.SelectedTime = Nothing

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnResetFilters_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles btnResetFilters.Click

        Try

            'Eliminem tots els filtres (excepte els de dates) i recarreguem els resultats

            timFrom.SelectedTime = Nothing
            timTo.SelectedTime = Nothing

            cmbPeriods.Text = ""
            cmbSections.Text = ""
            cmbServices.Text = ""

            cmbAnalyzers.Text = ""
            cmbOperators.Text = ""
            cmbPatients.Text = ""

            cmbReactive.Text = ""
            cmbControl.Text = ""
            cmbLevel.Text = ""

            InitializeResults(SEC_id_section)

            ShowTabs()

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "All filters reseted")

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnRefresh_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles btnRefresh.Click

        Try

            'Recuperem els resultats del Servei WCF

            InitializeResults(SEC_id_section)

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Results list refreshed")

            ApplyFilter()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnErrors_Click(sender As System.Object, e As System.Windows.RoutedEventArgs)

        Try

            'Mostrem els errors que s'han produit

            tabErrors.Focus()
            tabErrors.IsSelected = True

            Main.SaveLog(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, "Errors showed")

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Sub btnShowReport_Click(sender As System.Object, e As System.Windows.RoutedEventArgs) Handles btnShowReport.Click


        'Mostrem l'informe dels resultats que veiem en pantalla

        Try

            Dim report As New Results_report

            report.DataSource = New ObservableCollection(Of GG_results)(griResults.Items.OfType(Of GG_results))
            report.txtFilters.Value = CrearstrFilter()
            report.txtStatistics.Value = RecogerEstadisticas()

            If Main.logedUser.USE_logo IsNot Nothing Then
                report.logo.Value = Main.logedUser.USE_logo
            End If

            report.cabecera1.Value = Main.GetVariable("General", "Header1")
            report.cabecera2.Value = Main.GetVariable("General", "Header2")

            'Traduim les capçaleres de la taula
            report.txtAnalyzer.Value = My.Resources.Resource.msg_Analyzer
            report.txtDate.Value = My.Resources.Resource.msg_Date
            report.txtHigh.Value = My.Resources.Resource.msg_High
            report.txtHour.Value = My.Resources.Resource.msg_Hour
            report.txtLow.Value = My.Resources.Resource.msg_Low
            report.txtOperator.Value = My.Resources.Resource.msg_Operator
            report.txtPatient.Value = My.Resources.Resource.msg_Patient
            report.txtResult.Value = My.Resources.Resource.msg_Result
            report.txtService.Value = My.Resources.Resource.msg_Service

            report.pageInfoTextBox.Value = Replace(report.pageInfoTextBox.Value, "of", "de")

            Dim reportViewer As New ReportViewer(report)
            reportViewer.Show()

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

    End Sub

    Private Function CrearstrFilter() As String

        'Recollim els filtres que es troben actius abans de presentar l'informe

        Dim strFilter As String = ""

        Try

            If datFrom.SelectedDate.ToString <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_StartDate & ": " & Format(datFrom.SelectedDate, "dd/MM/yyyy") & vbCrLf
            End If

            If datTo.SelectedDate.ToString <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_EndDate & ": " & Format(datTo.SelectedDate, "dd/MM/yyyy") & vbCrLf
            End If

            If timFrom.SelectedTime.ToString <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_StartHour & ": " & Left(timFrom.SelectedTime.Value.ToString, Len(timFrom.SelectedTime.Value.ToString) - 3) & vbCrLf
            End If

            If timTo.SelectedTime.ToString <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_EndHour & ": " & Left(timTo.SelectedTime.Value.ToString, Len(timTo.SelectedTime.Value.ToString) - 3) & vbCrLf
            End If

            If cmbOperators.Text <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_Operator & ": " & cmbOperators.Text & vbCrLf
            End If

            If cmbAnalyzers.Text <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_Analyzer & ": " & cmbAnalyzers.Text & vbCrLf
            End If

            If cmbReactive.Text <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_Reactive & ": " & cmbReactive.Text & vbCrLf
            End If

            If cmbServices.Text <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_Service & ": " & cmbServices.Text & vbCrLf
            End If

            If cmbPatients.Text <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_Patient & ": " & cmbPatients.Text & vbCrLf
            End If

            If cmbControl.Text <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_Control & ": " & cmbControl.Text & vbCrLf
            End If

            If cmbLevel.Text <> "" Then
                strFilter = strFilter & My.Resources.Resource.msg_Level & ": " & cmbLevel.Text & vbCrLf
            End If

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return My.Resources.Resource.msg_ActiveFilters & " " & Main.obj_DAO.GetTests("TES_id_test=" & currentTest, "")(0).TES_name & vbCrLf & vbCrLf & strFilter

    End Function

    Private Function RecogerEstadisticas() As String

        'Presenta les estadístiques ( Total resultats / Mitja / Desviació / Coeficient variació )

        Dim strStatistics As String = ""

        Try

            strStatistics = btnTotal.Content & " | "
            strStatistics = strStatistics & btnAverage.Content & " | "
            strStatistics = strStatistics & btnSD.Content & " | "
            strStatistics = strStatistics & btnCV.Content

        Catch ex As Exception

            Main.ExceptionManagement(System.Reflection.MethodBase.GetCurrentMethod.Name.ToString, ex)

        End Try

        Return strStatistics

    End Function

End Class

Class StatisticsPresenter

    Private _STA_name As String
    Public Property STA_name() As String
        Get
            Return _STA_name
        End Get
        Set(ByVal value As String)
            _STA_name = value
        End Set
    End Property

    Private _STA_total As Integer
    Public Property STA_total() As Integer
        Get
            Return _STA_total
        End Get
        Set(ByVal value As Integer)
            _STA_total = value
        End Set
    End Property

End Class